package com.apelon.modules.dts.editor.tableviewer;


import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.net.URL;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JMenu;
import javax.swing.JMenuItem;

import com.apelon.apps.dts.editor.modules.DTSEditorModule;
import com.apelon.apps.dts.editor.modules.DTSEditorModuleMgr;
import com.apelon.beans.dts.plugin.connection.ConnectionCloseVeto;
import com.apelon.beans.dts.plugin.connection.DtsConnectionEvent;
import com.apelon.beans.dts.plugin.connection.DtsConnectionListener;


/**
 * Entry class for DTS Table Viewer plug-in module
 * <p>
 * Copyright (c) 2006 Apelon, Inc. All rights reserved.
 * @since 3.4.0
 */
public class TableViewer extends DTSEditorModule implements DtsConnectionListener
{
    private static final String TABLE_VIEWER_NAME = "Table Viewer";
    private static final String USER_GUIDE = "tablevieweruserguide.htm";
	private static final String TABLE_VIEWER_CONFIG_FILE = "tableviewerconfig.xml";
    private static final boolean debug = true;
    private DTSEditorModuleMgr moduleMgr;
    private JMenuItem tableViewerMenuItem = null;
    private JMenuItem tableViewerHelpMenuItem = null;

    public TableViewer () {
        if (debug) System.out.println ("TableViewer.TableViewer()");
    }

    /**
     * This method is called by DTS Editor to initialize the module.
     * @param mgr DTSEditorModuleMgr
     */
    public void initModule (DTSEditorModuleMgr mgr) {
        if (debug) System.out.println ("TableViewer.InitModule()");
        moduleMgr = mgr;
        initModulePanels();
        moduleMgr.registerConnectionListener (this);
    }

    public String getDTSModuleConfigFile() { return TABLE_VIEWER_CONFIG_FILE; }



    //panel stuff


    private void initModulePanels() {
        moduleMgr.getRightTabbedPane().add(TABLE_VIEWER_NAME,
                                           getTableViewerPanel());
    }

    TableViewerPanel tableViewerPanel = null;

    private TableViewerPanel getTableViewerPanel()
    {
        if (debug) System.out.println ("TableViewer.getTableViewerPanel()");
        if (tableViewerPanel==null) tableViewerPanel =
            new TableViewerPanel(moduleMgr, getDTSModuleConfig());
        return tableViewerPanel;
    }


    //required
    public JMenu[] getModuleMenus() { return null; }

    //required
    //  loads menu items declared by the plugin
    public JMenuItem[] getModuleMenuItems (int group) {
        JMenuItem[] menuItems;
        if (group == TOOLS_MENU_ITEMS) {
            menuItems = new JMenuItem[] { getTableViewerMenuItem() };
        }
        else if (group == HELP_MENU_ITEMS) {
            menuItems = new JMenuItem[] { getTableViewerHelpMenuItem() };
        }
        else menuItems = new JMenuItem[0];
        return menuItems;
    }

    public JComponent[] getModuleToolbarItems () {
        return null;
    }

    private JMenuItem getTableViewerMenuItem () {
        if (tableViewerMenuItem == null) {
            tableViewerMenuItem = new JMenuItem();
            tableViewerMenuItem.setSelected     (false);
            tableViewerMenuItem.setName         ("TableViewerMenuItem");
            tableViewerMenuItem.setText         ("New Table Viewer");
            tableViewerMenuItem.setToolTipText  ("Open New Table Viewer");
            tableViewerMenuItem.setEnabled      (false);
            tableViewerMenuItem.addActionListener (new ActionListener() {
                    public void actionPerformed (ActionEvent ae) {
                        tableViewerSelected(ae);
                    }
            });
        }
        return tableViewerMenuItem;
    }

    /** help is in panel menu bar */
    private JMenuItem getTableViewerHelpMenuItem () {
        if (tableViewerHelpMenuItem == null) {
            tableViewerHelpMenuItem = new JMenuItem();
            tableViewerHelpMenuItem.setSelected     (false);
            tableViewerHelpMenuItem.setName         ("TableViewerHelpMenuItem");
            tableViewerHelpMenuItem.setText         ("Table Viewer User Guide");
            tableViewerHelpMenuItem.setToolTipText  ("Open Table Viewer User Guide");
            tableViewerHelpMenuItem.setEnabled      (true);
            tableViewerHelpMenuItem.addActionListener (new ActionListener() {
                    public void actionPerformed (ActionEvent ae) {
                        URL url = getHelpURL();
                        if (url==null) {
                            System.out.println("TableViewer.error accessing help file.");
                            moduleMgr.showErrorMessage("Exception getting help file " +
                                                       USER_GUIDE +
                                                       ".");
                        }
                        else moduleMgr.showHelpPanel(url);
                    }
            });
        }
        return tableViewerHelpMenuItem;
    }

    static URL getHelpURL() {
        try {
            String helpFile = USER_GUIDE;
            StringBuffer urlSbf = new StringBuffer();
            File dir = new File(System.getProperty("user.dir"));
            String absPath = dir.getCanonicalPath();
            String sep = System.getProperty("file.separator");
            File parentFile = new File(absPath);
            String parent = parentFile.getParent();

            if (helpFile.trim().length() > 0)
            {
                urlSbf.append("file:").append(sep).append(sep).append(parent).
                        append(sep).append("docs").append(sep).
                        append("help").append(sep).append(helpFile);
                System.out.println("locating Help File: url = " + urlSbf.toString());
                return new URL(urlSbf.toString());
            }
            else
            {
                return null;
            }
        }
        catch (Exception e) {
            return null;
        }
    }



    //DtsConnectionListener interface

    public void connectionOpened(DtsConnectionEvent event){
        if (debug) System.out.println ("TableViewer.connectionOpened(): event received");
        enableModule(true);
    }

    public void connectionClosed(DtsConnectionEvent event){
        if (debug) System.out.println ("TableViewer.connectionClosed(): event received");
        enableModule(false);
    }

    public void connectionClosing(DtsConnectionEvent event) { }

    public void connectionWillClose(DtsConnectionEvent event, ConnectionCloseVeto veto) { }

    protected void enableModule(boolean connected) {
        tableViewerMenuItem.setEnabled(connected);
    }


    /** run the viewer */
    public void tableViewerSelected(ActionEvent ae)
    {
        System.out.println("TableViewer.starting floating Viewer panel");
        TableViewerPanel panel = new TableViewerPanel(moduleMgr, getDTSModuleConfig());
        JDialog dialog = moduleMgr.createDialog(panel, "Table Viewer", 500, 600);
        dialog.setModal(false);
        dialog.setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
        dialog.setVisible(true);
    }
}
