package com.apelon.modules.dts.editor.tableviewer;


import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Arrays;
import java.util.Vector;
import javax.swing.*;
import javax.swing.border.TitledBorder;

import com.apelon.beans.dts.plugin.DTSAppManager;
import com.apelon.dts.client.DTSQuery;
import com.apelon.dts.client.common.DTSObject;
import com.apelon.dts.client.concept.DTSSearchOptions;
import com.apelon.dts.client.namespace.Namespace;
import com.apelon.dts.dtsutil.QualifiedObject;
import org.apache.log4j.Category;


/**
 * display attributes for CodeSetTable
 * <p>
 * Copyright (c) 2006 Apelon, Inc. All rights reserved.
 * @since 3.4.0
 */
class TableViewerAttrDialog extends JDialog implements ActionListener 
{
    JList list;
    JButton selectButton, cancelButton;
    JLabel dispLabel;
    JComboBox attrCombo, dispCombo;
    DTSQuery queryMgr;
    String name;
    TableViewerAttribute selectedAttr = null;
        
    DTSSearchOptions sopts = new DTSSearchOptions(100, 
                                                  DTSSearchOptions.ALL_NAMESPACES);
        
    TableViewerAttrDialog(Frame owner, TableViewerTableDirector dir) {
        super(owner, " Select Concept Attribute ", true);    //modal
        setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
        queryMgr = DTSAppManager.getQuery();
        name = dir.getActiveName();
        int id = dir.getActiveNamespaceId();
                
        //Container pane = getContentPane();
        JPanel panel = new JPanel(new BorderLayout());
        panel.setBorder(new TitledBorder(dir.getDirectorLabel()+": "+name+" "));
        setContentPane(panel);
                
        //do layout
        GridBagLayout gb = new GridBagLayout();
        panel.setLayout(gb);
        GridBagConstraints gc = new GridBagConstraints();
        gc.anchor = GridBagConstraints.CENTER;
        gc.gridwidth = 1;
        gc.gridheight = 1;
                
        //first the labels
        gc.gridx = 0;
        gc.gridy = 0;
        gc.weightx = 0.0;
        gc.weighty = 0.0;
        gc.fill = GridBagConstraints.HORIZONTAL;
        gc.insets = new Insets(5, 5, 5, 5);     //top, left, bottom, right
        JLabel label = new JLabel("         Add Attribute: ", SwingConstants.RIGHT);
        //attrLabel.setAlignmentX(.5f);
        panel.add(label, gc);
        gc.gridy += 1;
        dispLabel = new JLabel("displaying: ", SwingConstants.RIGHT);
        dispLabel.setEnabled(false);
        //fieldsLabel.setAlignmentX(.5f);
        panel.add(dispLabel, gc);

        //now combos
        gc.gridx = 1;
        gc.gridy = 0;
        gc.weightx = 1.0;
        attrCombo = new JComboBox(getAttributes(id, ALL_ATTRIBUTES));
        attrCombo.addActionListener(this);
        panel.add(attrCombo, gc);
        gc.gridy++;
        dispCombo = new JComboBox();
        dispCombo.setEnabled(false);
        panel.add(dispCombo, gc);
                
        gc.gridx = 0;
        gc.gridy = 2;
        gc.gridwidth = 2;
                
        JPanel buttonPanel = new JPanel();
        selectButton = new JButton("Select");
        selectButton.addActionListener(this);
        buttonPanel.add(selectButton);
        cancelButton = new JButton("Cancel");
        cancelButton.addActionListener(this);
        buttonPanel.add(cancelButton);
        panel.add(buttonPanel, gc);

        pack();
        getRootPane().setDefaultButton(selectButton);
        //now make big enough for long names
        setSize(new Dimension(400, getSize().height));
    }
          
    public TableViewerAttribute getSelectedAttr() {
        return selectedAttr;
    }
          

    private final static Cursor busyCursor = new Cursor(Cursor.WAIT_CURSOR);
    private final static Cursor defCursor = new Cursor(Cursor.DEFAULT_CURSOR);

    /**
     * popup dialog at x, y (based on Component)
     *  on cancel disposes the dialog and returns)
     */
    public static TableViewerAttribute xshowDialog(Component comp, TableViewerTableDirector dir, 
                                                  int x, int y) {
        comp.setCursor(busyCursor);
        Frame frame = JOptionPane.getFrameForComponent(comp);
        TableViewerAttrDialog dialog = new TableViewerAttrDialog(frame, dir);
        Point pt = SwingUtilities.convertPoint(comp, x, y, frame);
        //System.out.println("From: "+x+","+y+" to "+pt.x+","+pt.y);
        dialog.setLocation(pt.x, pt.y);
        comp.setCursor(defCursor);
        dialog.setVisible(true);
        return dialog.getSelectedAttr();
    }
          
    /** popup centered on component */
    public static TableViewerAttribute showDialog(Component comp, TableViewerTableDirector dir) {
        comp.setCursor(busyCursor);
        Frame frame = JOptionPane.getFrameForComponent(comp);
        TableViewerAttrDialog dialog = new TableViewerAttrDialog(frame, dir);
        int x = comp.getLocation().x + 
            (comp.getSize().width -dialog.getSize().width)/2;
        int y = comp.getLocation().y + 
            (comp.getSize().height -dialog.getSize().height)/2;
        Point pt = SwingUtilities.convertPoint(comp, x, y, frame);
        dialog.setLocation(pt.x, pt.y);
        comp.setCursor(defCursor);
        dialog.setVisible(true);
        return dialog.getSelectedAttr();
    }
          


    /** button event handler */
    public void actionPerformed(ActionEvent ae) {
        Object src = ae.getSource();
        if (src==attrCombo) {
            //from attrCombo, see if a "reference" type (Role or Assn)
            QualifiedObject attr = (QualifiedObject)attrCombo.getSelectedItem();
            if ( TableViewerAttribute.isDirectAttribute(attr)) {
                dispLabel.setEnabled(false);
                dispCombo.setEnabled(false);
            }
            else {
                //need to get appropriate attributes for "referenced" concept
                //note we don't know what namespace they may be from
                dispCombo.setModel(new DefaultComboBoxModel(
                    getAttributes(-1, FIXED + PROPERTIES)));
                dispLabel.setEnabled(true);
                dispCombo.setEnabled(true);
            }
            return;
        }
        if (src==selectButton) {
            QualifiedObject attr = (QualifiedObject)attrCombo.getSelectedItem();
            //if indirect, add disp name
            if (dispCombo.isEnabled()) {
                QualifiedObject disp = (QualifiedObject)dispCombo.getSelectedItem();
                selectedAttr = new TableViewerAttribute(attr, disp);
            }
            else {
                selectedAttr = new TableViewerAttribute(attr);
            }
            //return;
        }
        setVisible(false);                         //done after either button
    }
          

         
    private static final int FIXED = 1;
    private static final int PROPERTIES = 2;
    private static final int ROLES = 4;
    private static final int ASSOCIATIONS = 8;
    private static final int ALL_ATTRIBUTES = FIXED + PROPERTIES + ROLES + 
        ASSOCIATIONS;


    /**
     * get an array of those attributes ( as QualifiedObjects) visible in namespace
     * id is namespace id or -1 if "all"
     * returns attributes in specified namespace plus all local namespace
     * flag specifies types of attributes to be returned 
     */
    public QualifiedObject[] getAttributes(int id, int flag) {
        Vector<QualifiedObject> vec;
        QualifiedObject[] attrArray = new QualifiedObject[0];
        int id1, start;
        DTSObject objects[];
         
        vec = new Vector<QualifiedObject>();
        if ((flag & FIXED)>0) {
            vec.addElement(new QualifiedObject(TableViewerAttribute.CONCEPT_NAME));
            vec.addElement(new QualifiedObject(TableViewerAttribute.CONCEPT_CODE));
            vec.addElement(new QualifiedObject(TableViewerAttribute.CONCEPT_ID));
            vec.addElement(new QualifiedObject(TableViewerAttribute.NAMESPACE));
            vec.addElement(new QualifiedObject(TableViewerAttribute.PREFERRED_NAME));
        }
        start = vec.size();             //don't sort before this
        try {
            //loop through the namespaces and get attributes requested
            Namespace[] spaces = queryMgr.getNamespaceQuery().getNamespaces();
            for (int ns=0; ns<spaces.length; ns++) {
                id1 = spaces[ns].getId();
                if (!(id<0 || id==id1 || spaces[ns].isLocal())) continue;       
                //now get "real" attributes
                if ((flag & PROPERTIES)>0) { 
                    objects = queryMgr.getNavQuery().getConceptPropertyTypes(id1);
                    saveObjects(objects, vec);
                }
                if ((flag & ROLES)>0) {
                    objects = queryMgr.getNavQuery().getRoleTypes(id1);
                    saveObjects(objects, vec);
                }
                if ((flag & ASSOCIATIONS)>0) {
                    objects = queryMgr.getAssociationQuery().
                        getConceptAssociationTypes(id1);
                    saveObjects(objects, vec);
                    saveInverseObjects(objects, vec);			//add inverses
                }
            }
        } catch (Exception e) {            
            Category.getInstance("tableviewer").error(e);
            System.out.println("Error retrieving attributes.");
        }
        attrArray = (QualifiedObject[])vec.toArray(attrArray);
        Arrays.sort(attrArray, start, attrArray.length);
        return attrArray;
    }


    /** save an array of DTSObjects into a Vector of QualifiedObjects */
    private void saveObjects(DTSObject[] objs, Vector<QualifiedObject> objVec) {
        for (int i=0; i<objs.length; i++) {
            objVec.addElement(new QualifiedObject(objs[i]));
        }
    }

    /** save an array of inverse DTSObjects (AssociationTypes) into a Vector of QualifiedObjects */
    private void saveInverseObjects(DTSObject[] objs, Vector<QualifiedObject> objVec) {
        for (int i=0; i<objs.length; i++) {
            objVec.addElement(new QualifiedObject(objs[i], true));
        }
    }

}