package com.apelon.modules.dts.editor.tableviewer;


import java.awt.Color;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.datatransfer.Transferable;
import java.awt.dnd.DnDConstants;
import java.awt.dnd.DropTarget;
import java.awt.dnd.DropTargetDragEvent;
import java.awt.dnd.DropTargetDropEvent;
import java.awt.dnd.DropTargetEvent;
import java.awt.dnd.DropTargetListener;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.EventListener;
import java.util.Vector;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.SwingConstants;
import javax.swing.SwingUtilities;
import javax.swing.event.EventListenerList;

import com.apelon.beans.dts.dnd.DtsDnd;
import com.apelon.dts.client.concept.DTSConcept;
import com.apelon.dts.dtsutil.QualifiedObject;
import com.apelon.beans.dts.plugin.transferable.DTSDataFlavor;
import com.apelon.beans.dts.plugin.transferable.DTSMultiData;
import org.apache.log4j.Category;


/**
 * panel to accept DnD concept representations
 * <p>
 * Copyright (c) 2006 Apelon, Inc. All rights reserved.
 * @since 3.4.0
 */
class TableViewerConceptAddPanel extends JPanel 
    implements ActionListener, DropTargetListener
{
    // constants for concept actions, these must agree!!
    final static String[] actions = 
    {       "+ Concept",
            "+ Children",
            "+ Concept & Children",
            "+ Descendants",
            "+ Concept & Descendants",
            "- Concept",
            "- Children",
            "- Concept & Children",
            "- Descendants",
            "- Concept & Descendants"
    };

    //code is SelectedIndex() + 1; note code%2=0 means include CONCEPT      
    final static int CONCEPT = 0;   
    final static int CHILDREN = 1;  
    final static int CONCEPT_AND_CHILDREN = 2;      
    final static int DESCENDANTS = 3;       
    final static int CONCEPT_AND_DESCENDANTS = 4;
    final static int DELETE = 5;      //used as modulo        
        
    JButton okButton, cancelButton;
    JTextField conceptField;
    JComboBox actionCombo;
    DTSConcept[] results = null;                    //results
        
    TableViewerConceptAddPanel()
    {
        this(null);
    } 
        
    TableViewerConceptAddPanel(DropTargetDropEvent de) 
    {
        super();
        //do layout
        GridBagLayout gb = new GridBagLayout();
        setLayout(gb);
        GridBagConstraints gc = new GridBagConstraints();
        gc.anchor = GridBagConstraints.CENTER;
        gc.gridwidth = 2;
        gc.gridheight = 1;
                
        //first the labels
        gc.gridx = 0;
        gc.gridy = 0;
        gc.weightx = 0.0;
        gc.weighty = 0.0;
        gc.fill = GridBagConstraints.HORIZONTAL;
        gc.insets = new Insets(10, 5, 5, 5);    //top, left, bottom, right
        JLabel label = new JLabel(" Drag Concept(s) to be added here: ", 
                                  SwingConstants.CENTER);
        //attrLabel.setAlignmentX(.5f);
        add(label, gc);
                
        gc.insets = new Insets(5, 5, 5, 5);             //top, left, bottom, right
        gc.gridy++;
        conceptField = new JTextField(100);
        conceptField.setEditable(false);
        add(conceptField, gc);
                
        gc.gridy++;
        gc.gridx = 0;
        gc.gridwidth = 1;
        label = new JLabel("Action: ", SwingConstants.RIGHT);
        add(label, gc);
        gc.gridx++;
        actionCombo = new JComboBox(actions);
        add(actionCombo, gc);
                
        gc.gridy++;
        gc.gridx = 0;
        gc.gridwidth = 2;
        JPanel buttonPanel = new JPanel();
        okButton = new JButton(" OK ");
        okButton.addActionListener(this);
        buttonPanel.add(okButton);
        cancelButton = new JButton(" Cancel ");
        cancelButton.addActionListener(this);
        buttonPanel.add(cancelButton);
        add(buttonPanel, gc);

        dropTarget = new DropTarget(conceptField, DnDConstants.ACTION_MOVE, 
                                    this, true);

        //now make big enough for long names
        setPreferredSize(new Dimension(400, 0));
        loadResults(de);            //process possible input
    }
          
    public DTSConcept[] getConcepts()
    {
        return results;
    }
          
    
    /**
     * load the Concept results (and display field) from a drop event 
     * (or constructor)
     * returns true on valid data
     */
    private boolean loadResults(DropTargetDropEvent de)
    {
        results = new DTSConcept[0];                   //clear results
        String s = "";
        if (de!=null) {
            try {
            	Transferable trans = de.getTransferable();
            	if (trans.isDataFlavorSupported(DTSDataFlavor.multiFlavor)) {
                    System.out.println("TableViewer.multi-drop");
                    Vector<Object> vec = (Vector<Object>)trans.getTransferData(DTSDataFlavor.multiFlavor);
                    if (vec.get(0) instanceof DTSConcept) {       
                    	//is a concept (assume the rest are as well)
                        results = (DTSConcept[])vec.toArray(results);
                        s = "(multiple)";
                    }
            	}
            	else {
            		//not a multi
                     System.out.println("TableViewer.single drop");
                 	if (trans.isDataFlavorSupported(DTSDataFlavor.conceptFlavor)) {
                        DTSConcept con = (DTSConcept)trans.getTransferData(DTSDataFlavor.conceptFlavor);
                        if (con!=null) {       
                        	//is a concept!
                            results = new DTSConcept[1];
                            results[0] = con;
                            s = QualifiedObject.getNameForObject(con);
                        }
                 	}
            		
            	}
                System.out.println(
                     "TableViewer.Drop recognized in ConceptAddPanel: "+ s);
            } catch (Exception e)
            {
                Category.getInstance("tableviewer").error(e);
                e.printStackTrace();
            }
        }
        conceptField.setBackground(Color.white);
        conceptField.setText(s);
        conceptField.setCaretPosition(0);
        return (!s.equals(""));
    }



    //button event handler
          
    private final static Cursor busyCursor = new Cursor(Cursor.WAIT_CURSOR);
    private final static Cursor defCursor = new Cursor(Cursor.DEFAULT_CURSOR);

    public void actionPerformed(ActionEvent ae)
    {
        if (ae.getSource()==okButton)
        {
            String name = conceptField.getText();   //actually now ignored, but ...
            if (!name.equals(""))
            {
                setCursor(busyCursor);
                int id = actionCombo.getSelectedIndex();
                //System.out.println("firing "+ id);
                fireActionEvent(new ActionEvent(this, id, name));
                setCursor(defCursor);
                conceptField.setText("");                       //reset
            }
        }
        else                                            //cancel, done
        {
            SwingUtilities.getWindowAncestor(this).dispose();
        }
    }
          

    //action listeners
    EventListenerList listenerList = new EventListenerList();
         
    public void addActionListener(ActionListener al)
    {
        listenerList.add(ActionListener.class, al);
    }
         
    public void removeActionListener(ActionListener al)
    {
        listenerList.remove(ActionListener.class, al);
    }
         
    protected void fireActionEvent(ActionEvent ae)
    {
        EventListener[] listeners = listenerList.getListeners(ActionListener.class);
        for (int i=listeners.length-1; i>=0; i--)
            ((ActionListener)listeners[i]).actionPerformed(ae);
    }
         

         
    DropTarget dropTarget;
         
    /** DropTargetListener interface */
    public void dragEnter(DropTargetDragEvent de)
    {
        conceptField.setBackground(Color.yellow);
        de.acceptDrag(DnDConstants.ACTION_MOVE);
    }
         
    public void drop(DropTargetDropEvent de)
    {
        if (loadResults(de))
        {
            de.acceptDrop(DnDConstants.ACTION_MOVE);
            de.dropComplete(true);
            return;
        }

        de.rejectDrop();
    }
         
    public void dragExit(DropTargetEvent de) 
    {
        conceptField.setBackground(Color.white);
    }
    public void dragOver(DropTargetDragEvent de) {}
    public void dropActionChanged(DropTargetDragEvent de) {}
}