package com.apelon.modules.dts.editor.tableviewer;

import com.apelon.apps.dts.editor.modules.DTSModuleConfig;
        
/**
 * manage Table Viewer configuration via DTSModuleConfig file
 * <p>
 * Copyright (c) 2006 Apelon, Inc. All rights reserved.
 * @since 3.4.0
 */

public class TableViewerConfig  {
	
    private static final boolean debug = false;
	private static DTSModuleConfig config;
 
    TableViewerConfig(DTSModuleConfig cfg) {
		config = cfg;
    }
        
    /** save the current file */
    boolean save() {
        try {
            config.saveProperties();
            if (debug) System.out.println("TableViewer.configuration saved.");
            return true;
        }
        catch (Exception e) {
            System.out.println("TableViewer.error saving configuration.");
            e.printStackTrace();
            return false;
        }
    }
        
    /** get an entry */
    String getEntry(String key) {
        return decode(config.getProperty(key));
    }

    /** update a config entry */
    boolean updateEntry(String key, String val) {
        config.storeProperty(key, encode(val));
        return save();
    }

    /** add a new entry */
    boolean addEntry(String key, String val) {
        config.storeProperty(key, encode(val));
        return save();
    }
    
    /***************************************************************/
    //value entity encoding and decoding - special version for table viewer
    //don't save an entity-based delimiter string in the property file
        
    //private static String plainStr[] = { "&", "<", ">", "\"" };	//"&" must be first
    //private static String cipherStr[] = { "&amp;", "&lt;", "&gt;", "&quot;" };
    private static String plainStr[] = { "&", "->" };			//"&" must be first
    private static String cipherStr[] = { "&amp;", "--" };
        
    private static String encode(String in) {
        String str = in;
        for (int i=0; i<plainStr.length; i++) str = replace(str, 
                  plainStr[i], cipherStr[i]);
        return str;
    }
        
    private static String decode(String in) {
    	if (in==null) return null;
        String str = in;
        for (int i=0; i<plainStr.length; i++) 
            str = replace(str, cipherStr[i], plainStr[i]);
        return str;
    }
        
    private static String replace(String str, String in, String out) {
        int i=-1;
        String s = str;
        while ((i=s.indexOf(in,++i))>=0) {
            s = s.substring(0,i)+out+s.substring(i+in.length());
            //System.out.println("'"+str+"' '"+s+"'");
        }
        return s;
    }
}
