package com.apelon.modules.dts.editor.tableviewer;

import javax.swing.border.EmptyBorder;

import com.apelon.apps.dts.editor.modules.DTSEditorModuleMgr;
import com.apelon.beans.dts.plugin.DTSAppManager;
import com.apelon.dts.client.DTSQuery;

import java.awt.BorderLayout;
import java.awt.Cursor;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.Window;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.SwingConstants;
import javax.swing.SwingUtilities;

/**
 * super class of director panel, implements common functions
 * Copyright (c) 2008 Apelon, Inc. All rights reserved.
 * @author jbowie
 */
public class TableViewerDirectorPanel extends JPanel implements ActionListener {

	/** function values */
	static final int INACTIVE = 0;
	static final int LOAD = 1;					//bit mask for LOAD
	static final int OPEN = 2;					//bit mask for OPEN
	
	protected final static String SELECT = "- Select -";
    protected final static Cursor busyCursor = new Cursor(Cursor.WAIT_CURSOR);
    protected final static Cursor defCursor = new Cursor(Cursor.DEFAULT_CURSOR);

    TableViewerTablePanel tablePanel;
    DTSEditorModuleMgr moduleMgr;               //main panel component
    DTSQuery queryMgr;
    int activeFunction;								//if active, selected function
    String activeName;								//if active, selected name
    JComboBox combo;
    JButton loadButton, openButton;

    TableViewerDirectorPanel(DTSEditorModuleMgr mgr, TableViewerTablePanel tpanel, int options) {
		super(new BorderLayout());
        tablePanel = tpanel;
        moduleMgr = mgr;
        queryMgr = DTSAppManager.getQuery();
        activeName = null;
        activeFunction = INACTIVE;
        
        //build the UI
        setBorder(new EmptyBorder(10, 5, 5, 5));
        JPanel panel = new JPanel();
        GridBagLayout gb = new GridBagLayout();
        panel.setLayout(gb);
        GridBagConstraints gc = new GridBagConstraints();
        gc.anchor = GridBagConstraints.WEST;
        gc.fill = GridBagConstraints.BOTH;
        gc.gridwidth = 1;
        gc.gridheight = 1;
           
        //label field
        gc.gridx = 0;
        gc.gridy = 0;
        gc.weightx = 0.0;
        gc.weighty = 0.0;
        gc.insets = new Insets(0, 5, 5, 5);     //top, left, bottom, right
        JLabel label = new JLabel(getDirectorLabel()+":", SwingConstants.RIGHT);
        panel.add(label, gc);
        gc.gridx++;
        gc.gridy = 0;
        gc.weightx = 1.0;
        combo = new JComboBox();
        combo.setEditable(false);
        combo.setMaximumRowCount(15);
        combo.addActionListener(this);
        panel.add(combo, gc);
        //**********
        gc.gridx++;
        gc.weightx = 0.0;
        //try to match button widths
        openButton = new JButton((options&OPEN)>0?" Open ":"           ");
        if ((options&OPEN)>0) {
        	openButton.setToolTipText("Click to open the table with no concepts.");
            openButton.addActionListener( new ActionListener() {
                public void actionPerformed(ActionEvent ae) {
                    if (okayToContinue(OPEN)) load((String)combo.getSelectedItem(), OPEN);
                }
            });
        }
        openButton.setEnabled(false);
        openButton.setBorderPainted((options&OPEN)>0);
        openButton.setContentAreaFilled((options&OPEN)>0);
        panel.add(openButton, gc);
        gc.gridx++;
        loadButton = new JButton((options&LOAD)>0?" Load ":"");
        if ((options&LOAD)>0) {
        	loadButton.setToolTipText("Click to load the table with all "+getDirectorLabel()+" concepts.");
            loadButton.addActionListener( new ActionListener() {
                public void actionPerformed(ActionEvent ae) {
                    if (okayToContinue(LOAD)) load((String)combo.getSelectedItem(), LOAD);
                }
            });
        }
        loadButton.setEnabled(false);
        loadButton.setBorderPainted((options&LOAD)>0);
        loadButton.setContentAreaFilled((options&LOAD)>0);
        panel.add(loadButton, gc);
        add(panel, "Center");
	}
    
    /** load the table and set active variables as appropriate */
    private void load(String name, int func) {
    	if (loadTable(name, func)) {
    		activeName = name;
    		activeFunction = func;
    	}
    	else {
            tablePanel.enableModule(true);	//error, clear out the table panel
    		activeName = "";
    		activeFunction = INACTIVE;
    	}
    	tablePanel.setTitle();				//finally set the title since activeName is set
    }
    
    /** test if table can be reset/reloaded */
    protected boolean okayToContinue(int fnc) {
        if (tablePanel.isLoaded()) {
        	Window win = SwingUtilities.getWindowAncestor(this);
            if (JOptionPane.showConfirmDialog(win,
                "Are you sure you want to "+(fnc==LOAD?"overwrite":"clear")+" existing table data?",
                "Table Viewer", JOptionPane.YES_NO_OPTION)!=JOptionPane.YES_OPTION)
                return false;
        }
        return true;
    }

	/** get the active function: LOAD/OPEN */
	public int getActiveFunction() { return activeFunction; }
	
	/** get the active name */
	public String getActiveName() { return activeName; }
	
	/** enable the module */
    void enableModule(boolean enable) {
        //load the combo
        if (enable) {
        	loadKeyCombo();
        }
        else {
            ((DefaultComboBoxModel)combo.getModel()).removeAllElements();
        }
        openButton.setEnabled(false);
        loadButton.setEnabled(false);
    }

    /** load the key combo on enable */
    protected void loadKeyCombo() { loadKeyCombo("", ""); }

    /** (re)load the key combo on enable or group add/delete/change */
    protected void loadKeyCombo(String oldName, String newName) {
        DefaultComboBoxModel model; 
        String key = (String)combo.getSelectedItem();		//save current value, could be null or SELECT
    	boolean active = key==activeName;					//true if selected is active
    	if (key!=null && key.equals(oldName)) key = newName;	//update the name if a name change
    	combo.removeActionListener(this);
    	String[] vals = getDirectorNames();
    	if (vals.length==0) {
            model = new DefaultComboBoxModel();
    	}
    	else {
    		//add SELECT
    		String[] options = new String[vals.length+1];
    		options[0] = SELECT;
    		for (int i=0; i<vals.length; i++) options[i+1] = vals[i];
            model = new DefaultComboBoxModel(options);
    	}
        combo.setModel(model);
        //set the selected item
        if (key==null) combo.setSelectedIndex(combo.getItemCount()==0?-1:0);
        else if (key.equals("")) {					//deleted element			
            if (active) {
	            moduleMgr.showErrorMessage(getDirectorLabel()+" '"+oldName+
	            		"' has been deleted. Table will be reset.");
	            tablePanel.enableModule(true);			//clear out the table panel
	    		activeName = "";						//nothing active
	    		activeFunction = INACTIVE;
	            openButton.setEnabled(false);
	            loadButton.setEnabled(false);
            }
            combo.setSelectedIndex(combo.getItemCount()==0?-1:0);	//reset the combo
        }
        else {
        	combo.setSelectedItem(key);
        	if (active) activeName = key;				//could have changed
        }
        tablePanel.setTitle();							//update the title
        combo.addActionListener(this);
    }

    /*****************************************************************/
    //key combo action listener
    public void actionPerformed(ActionEvent ae) {
    	boolean b = combo.getSelectedIndex()>0;			//ignore SELECT
        openButton.setEnabled(b);    //enable ops after valid selection
        loadButton.setEnabled(b);
    }
    
    /*****************************************************************/
   //methods to be overridden
    
    /** get the label for this director */
    protected String getDirectorLabel() { return "Director"; }
     
    /** get the names for the selection combo */
    protected String[] getDirectorNames() { return new String[0]; }
    
    /** load the table, return success */
    protected boolean loadTable(String name, int func) { return false; }
    
}
