package com.apelon.modules.dts.editor.tableviewer;


import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.PrintWriter;
import java.util.Enumeration;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableColumn;
import javax.swing.table.TableColumnModel;

import org.apache.log4j.Category;


/**
 * export code set table to text or XML file
 * <p>
 * Copyright (c) 2006 Apelon, Inc. All rights reserved.
 * @since 3.4.0
 */
class TableViewerExport {
 
    private static final String colDelim = "\t";
        
    private TableViewerExport() { }
 
    public static boolean export(TableViewerTable exporttable, String exportfile) 
    {
        PrintWriter outWriter = null;
        StringBuffer resBuf = new StringBuffer();
        boolean success = false;
        boolean xml = getFileExtension(exportfile).equals("xml");

        TableViewerTable table = exporttable;
        DefaultTableModel model = (DefaultTableModel)table.getModel();
                
        //build column headers
        TableColumnModel colModel = table.getColumnModel();
        int colCount = colModel.getColumnCount();
        String[] headers = new String[colCount];
        Enumeration col_enum = colModel.getColumns();
        for (int i=0; col_enum.hasMoreElements(); i++)
            headers[i] = (String)
                ((TableColumn)col_enum.nextElement()).getHeaderValue();
                
        try {
            //set up log file
            outWriter = new PrintWriter(new BufferedWriter(
                  new FileWriter(exportfile)));
            //get column info;
            if (xml) outWriter.println("<DTSExport>");
            else outWriter.println(getHeaderLine(headers));
            for (int row=0, n=model.getRowCount(); row<n; row++)
            {
                resBuf.delete(0, resBuf.length());
                if (xml) resBuf.append("  <concept>\n");
                for (int col=0; col<colCount; col++)
                {
                    if (xml) 
                    { 
                        resBuf.append("    <field name=\"");
                        resBuf.append(headers[col]);
                        resBuf.append("\" value=\"");
                    }
                    else
                        if (col>0) resBuf.append(colDelim);
                                                
                    resBuf.append(model.getValueAt(row, 
                        table.convertColumnIndexToModel(col)));
                    if (xml) resBuf.append("\" />\n");
                }
                                
                //write the row
                if (xml) resBuf.append("  </concept>\n");
                outWriter.println(resBuf.toString());
            }
            if (xml) outWriter.println("</DTSExport>");
            success = true;
        } catch (Exception e) {
            Category.getInstance("tableviewer").error(e);
            e.printStackTrace();
        }
        finally {
            if (outWriter!=null) outWriter.close();
            outWriter = null;
        }
        return success;
    }
        
    private static String getHeaderLine(String[] headers) {
        StringBuffer sb = new StringBuffer();
        for (int col=0; col<headers.length; col++) {
            if (col>0) sb.append(colDelim);
            sb.append(headers[col]);
        }
        return sb.toString();   
    }

    /** get a file extension */
    private static String getFileExtension(String filen) {
        int i = filen.lastIndexOf('.');
        if(i>0 && i<filen.length()-1) 
            return filen.substring(i+1).toLowerCase();
        return "";      
    }
}