package com.apelon.modules.dts.editor.tableviewer;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;
import javax.swing.JTextField;
import javax.swing.SwingConstants;
import javax.swing.SwingUtilities;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;

import java.util.ArrayList;

/**
 * Find/Replace dialog for Table Viewer
 * <p>
 * Copyright (c) 2006 Apelon, Inc. All rights reserved.
 * @since 3.4.0
 */

public class TableViewerFindDialog extends JDialog {
	
	private static int WIDTH = 400;
	private static int HEIGHT = 250;
	
	private FindReplacePanel findPanel;
	private FindReplacePanel replacePanel;
	private JTabbedPane tabPane;
	private TableViewerTablePanel tablePanel;
	private TableViewerTable table;
	
	TableViewerFindDialog(TableViewerTablePanel panel) {
		super();
		setTitle("Find and Replace");
		JPanel content = new JPanel(new BorderLayout());
		tablePanel = panel;
		tabPane = new JTabbedPane();
		findPanel = new FindReplacePanel(false);
		tabPane.add("Find", findPanel);
		replacePanel = new FindReplacePanel(true);
		tabPane.add("Replace", replacePanel);
		content.add(tabPane, "Center");
		setContentPane(content);
		setDefaultCloseOperation(JDialog.HIDE_ON_CLOSE);
		setModal(false);
		pack();				//lay it out now
		//fix size so buttons don't move
		setSize(new Dimension(WIDTH, HEIGHT));
	}
	
	/** load the panel's attribute select combos */
	void load(TableViewerAttribute[] attrs) {
		table = tablePanel.viewTable;			//get the table
		findPanel.load(attrs);
		replacePanel.load(attrs);
	}
	
	/** show the find dialog centered in parent panel */
	public void show() {
		Point pt = new Point();
		pt.x = (tablePanel.getWidth()-WIDTH)/2;
		pt.y = (tablePanel.getHeight()-HEIGHT)/2;
		pt = SwingUtilities.convertPoint(tablePanel, pt, null);
		setLocation(pt.x, pt.y);
		super.show();
		((FindReplacePanel)tabPane.getSelectedComponent()).findText.requestFocus();
	}
	
	/************************************************************/
	//inner class for the two panels
	class FindReplacePanel extends JPanel implements ActionListener{
		
		JButton findButton, replaceButton, repAllButton;
		JTextField findText, replaceText;
		JComboBox attrBox;
		JCheckBox caseChk, allChk;
		int colIndex[];					//column index by attr combo index
		boolean isReplace;
		
		FindReplacePanel(boolean b) {
			super(new BorderLayout());
			isReplace = b;
			JPanel panel = new JPanel(new GridBagLayout());
			GridBagConstraints gc = new GridBagConstraints();
			gc.anchor = GridBagConstraints.CENTER;
			gc.gridwidth = 1;
			gc.gridheight = 1;
			gc.weightx = 0.0;
			gc.weighty = 0.0;
			gc.fill = GridBagConstraints.HORIZONTAL;
			gc.insets = new Insets(5, 5, 5, 5);	//top, left, bottom, right
			gc.gridx = gc.gridy = 0;
			//spaces to try to match "Replace Text" width
			panel.add(new JLabel("      Find Text:", SwingConstants.RIGHT), gc);
			gc.gridx = 1;
			gc.weightx = 1.0;
			findText = new JTextField();
			//add a listener to track changes to text
			findText.getDocument().addDocumentListener(new DocumentListener() {
				public void changedUpdate(DocumentEvent de) { resetSearch(); }
				public void removeUpdate(DocumentEvent de) { resetSearch(); }
				public void insertUpdate(DocumentEvent de) { resetSearch(); }
			});
			panel.add(findText, gc);
			gc.gridx = 2;
			gc.weightx = 0.0;
			if (!isReplace) {
				findButton = new JButton("Find Next");
				findButton.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent ae) {
						findNext(true);
					}
				});
				panel.add(findButton, gc);
			}
			else {
				replaceButton = new JButton("Replace Next");
				replaceButton.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent ae) {
						replaceNext(true);
					}
				});
				panel.add(replaceButton, gc);
				gc.gridx = 0;
				gc.gridy++;
				panel.add(new JLabel("Replace Text:", SwingConstants.RIGHT), gc);
				gc.gridx = 1;
				gc.weightx = 1.0;
				replaceText = new JTextField();
				replaceText.getDocument().addDocumentListener(new DocumentListener() {
					public void changedUpdate(DocumentEvent de) { resetSearch(); }
					public void removeUpdate(DocumentEvent de) { resetSearch(); }
					public void insertUpdate(DocumentEvent de) { resetSearch(); }
				});
				panel.add(replaceText, gc);
				gc.gridx++;
				gc.weightx = 0.0;
				repAllButton = new JButton("Replace All");
				repAllButton.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent ae) {
						replaceAll();
					}
				});
				panel.add(repAllButton, gc);
			}
			//finish with close
			gc.gridx = 2;
			gc.gridy++;
			JButton closeButton = new JButton("Close");
			closeButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent ae) {
					TableViewerFindDialog.this.hide();
				}
			});
			panel.add(closeButton, gc);
			add(panel, "North");
			//now bottom panel
			panel = new JPanel(new GridBagLayout());
			gc = new GridBagConstraints();
			gc.anchor = GridBagConstraints.WEST;
			gc.gridwidth = 1;
			gc.gridheight = 1;
			gc.weightx = 0.0;
			gc.weighty = 0.0;
			gc.fill = GridBagConstraints.HORIZONTAL;
			gc.insets = new Insets(5, 5, 5, 5);	//top, left, bottom, right
			gc.gridx = gc.gridy = 0;
			attrBox = new JComboBox();
			panel.add(attrBox, gc);
			gc.gridx = 1;
			gc.weightx = 1.0;
			caseChk = new JCheckBox("Match case");
			caseChk.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent ae) {
					resetSearch();
				}
			});
			panel.add(caseChk, gc);
			gc.gridy++;
			allChk = new JCheckBox("Match entire cell contents");
			allChk.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent ae) {
					resetSearch();
				}
			});
			panel.add(allChk, gc);
			add(panel, "South");
		}
		
		//load the combo box with a column list
		void load(TableViewerAttribute[] attrs) {
			String[] findList = null;
			if (isReplace) {
				ArrayList repList = new ArrayList();
				TableViewerAttribute attr;
				for (int i=1; i<attrs.length; i++) {
					attr = attrs[i];
					if (table.isCellEditable(0,i-1)) repList.add(attr.toString());
				}
				findList = (String[])repList.toArray(new String[0]);
				colIndex = new int[findList.length];
				//now fill colIndex with the relevant column indices
				for (int i=0; i<findList.length; i++)
					for (int j=1; j<attrs.length; j++)
						if (findList[i].equals(attrs[j].toString())) {
							colIndex[i] = j-1;
							break;
						}
			}
			else {
				findList = new String[attrs.length];
				colIndex = new int[attrs.length];
				findList[0] = "All columns";
				colIndex[0] = -1;
				//skip first hidden column
				for (int i=1; i<attrs.length; i++) {
					findList[i] = attrs[i].toString();
					colIndex[i] = i-1;					
				}
			}
			attrBox.removeActionListener(this);
			attrBox.setModel(new DefaultComboBoxModel(findList));
			attrBox.addActionListener(this);
			enablePanel(findList.length!=0);
			initSearch();
		}
		
		//listen to attrBox changes for search reset
		public void actionPerformed(ActionEvent ae) {
			resetSearch();
		}
		
		private void enablePanel(boolean b) {
			findText.setEnabled(b);
			attrBox.setEnabled(b);
			caseChk.setEnabled(b);
			allChk.setEnabled(b);
			if (isReplace) {
				replaceButton.setEnabled(b);
				repAllButton.setEnabled(b);
				replaceText.setEnabled(b);
			} 
			else {
				findButton.setEnabled(b);
			}
		}
		
		/************************************************************/
		//action routines for find and replace
		//object for search location
		class Location {
			int row;
			int col;
			
			public String toString() { return row+":"+col; }
			
			public boolean equals(Object loc) {
				if (!(loc instanceof Location)) return false;
				Location myloc = (Location)loc;
				return (myloc.row==row && myloc.col==col);
			}
		}
		
		Location loc = new Location();		//current location
		Location start = new Location();	//starting location for sequence
		boolean newSearch;					//true to (re)start search
		
		//initialize the find/replace search to selection or start of table
		private void initSearch() {
			if (table.getSelectedRow()<0)
				loc.row = loc.col = 0;
			else {
				loc.row = table.getSelectedRow();
				loc.col = table.getSelectedColumn();
			}
			//must do below after dialog has been realized
			//only needs to be done once, but ...
			if (isReplace) replaceButton.setPreferredSize(replaceButton.getSize());
			else findButton.setPreferredSize(findButton.getSize());
			resetSearch();
		}
		
		//set a starting search location from loc (to avoid wrapping)
		private void resetSearch() {
			if (isReplace) replaceButton.setText("Replace");
			else findButton.setText("Find");
			newSearch = true;
		}
		
		//continue a search
		private void continueSearch() {
			if (isReplace) replaceButton.setText("Replace Next");
			else findButton.setText("Find Next");
		}
		
		//find the next occurence of the search string
		//returns position or <0 if not found or error;
		private int findNext(boolean report) {
			String val;
			int p;
			boolean noCase = !caseChk.isSelected();
			String str = findText.getText();
			if (str.equals("")) {
				tablePanel.moduleMgr.showErrorMessage("Empty find string!");
				return -2;
			}
			if (noCase) str = str.toLowerCase();
			int column = colIndex[attrBox.getSelectedIndex()];	//get search column
			while (true) {
				//if an initial search, set first cell
				if (newSearch) {
					if (table.getSelectedRow()<0)
						loc.row = loc.col = 0;
					else {
						loc.row = table.getSelectedRow();
						loc.col = table.getSelectedColumn();
					}
					if (column>=0) loc.col = column;
					start.row = loc.row;
					start.col = loc.col;
				}
				else {
					//move to next cell, scan columns first
					if (column<0) {
						loc.col++;				//move across columns
						if (loc.col==table.getColumnCount()) {
							loc.col = 0;
							loc.row++;
						}
					}
					else loc.row++;
					if (loc.row==table.getRowCount()) loc.row = 0;
					if (loc.equals(start)) break; //done
				}
				newSearch = false;
				val = (String)table.getValueAt(loc.row, loc.col);
				if (noCase) val = val.toLowerCase();
				p = val.indexOf(str);
				if (p>=0 && (!allChk.isSelected() ||val.length()==str.length())) {
					//found a match, mark it
					table.changeSelection(loc.row, loc.col, false, false);
		            continueSearch();
		            return p;
				}
				//no match at current cell, move on
			}  
			//search string not found
			if (report)
				JOptionPane.showMessageDialog(getParent(), 
                    "'"+findText.getText()+"' not found.", 
                    "Table Viewer Find", JOptionPane.INFORMATION_MESSAGE);
            resetSearch();
            return -1;
		}
		
		//replace the next occurence, returns true if replacement made
		private boolean replaceNext(boolean report) {
			int p = findNext(report);
			if (p>=0) {
				String v = (String)table.getValueAt(loc.row, loc.col);
				v = v.substring(0,p)+replaceText.getText()+
									v.substring(p+findText.getText().length());
				table.setValueAt(v, loc.row, loc.col);
				//System.out.println("setting "+loc+" to '"+v+"'");
				return true;
			}
			return false;
		}
		
		private void replaceAll() {
			int cnt = 0;
			while (replaceNext(false)) cnt++;
            JOptionPane.showMessageDialog(getParent(), 
                    cnt+" replacement"+(cnt==1?"":"s")+" made.", 
                    "Table Viewer Replace", JOptionPane.INFORMATION_MESSAGE);
		}
		
	}

}
