package com.apelon.modules.dts.editor.tableviewer;


import java.util.Vector;
import javax.swing.table.DefaultTableModel;

import com.apelon.apps.dts.editor.modules.DTSEditorModuleMgr;
import com.apelon.beans.dts.plugin.DTSAppManager;
import com.apelon.dts.client.DTSQuery;
import com.apelon.dts.client.association.AssociationType;
import com.apelon.dts.client.association.ConceptAssociation;
import com.apelon.dts.client.attribute.DTSProperty;
import com.apelon.dts.client.attribute.DTSPropertyType;
import com.apelon.dts.client.concept.ConceptAttributeSetDescriptor;
import com.apelon.dts.client.concept.DTSConcept;
import com.apelon.dts.dtsutil.QualifiedObject;
import org.apache.log4j.Category;

        
/**
 * data model for Table Viewer package
 * <p>
 * Copyright (c) 2006 Apelon, Inc. All rights reserved.
 * @since 3.4.0
 */
class TableViewerModel extends DefaultTableModel {
    DTSEditorModuleMgr moduleMgr;          //main panel component
        
    TableViewerModel(DTSEditorModuleMgr mgr) {
        super();
        moduleMgr = mgr;
    }

    /** editable if local DTSPropertyType or local indirect NAME attribute */
    public boolean isCellEditable(int row, int col) {
        TableViewerAttribute attr = getColumnIdentifier(col);
        if (!attr.isLocal()) return false;
        if (attr.getAttribute().getObject() instanceof DTSPropertyType) return true;
        if (attr.getDisplayAttribute()!=null && 
            attr.getDisplayAttribute().getName().equals(
                                       TableViewerAttribute.CONCEPT_NAME))
            return true;
        return false;
    }
         
    public TableViewerAttribute getColumnIdentifier(int col) {
        return (TableViewerAttribute)columnIdentifiers.get(col);
    }
         
    
    /**
     * set new cell value
     * attribute must be either a ConceptAssociation NAME or DTSPropertyType
     * setting an empty value deletes the attribute
     */
    public void setValueAt(Object val, int row, int col) {
        DTSProperty props[];
        ConceptAssociation assns[];
        int i;
        boolean success = true;
        boolean update = false;                 //flag for delayed row redisplay
                
        DTSQuery queryMgr = DTSAppManager.getQuery();
        String oldVal = (String)getValueAt(row, col);
        if (oldVal.equals("") && val.equals("")) return;        //nothing to do
        DTSConcept con = (DTSConcept)getValueAt(row, 0);        //base concept
        TableViewerAttribute attr = getColumnIdentifier(col);
        if (attr.getAttribute().getObject() instanceof DTSPropertyType) {
            try {   
                DTSPropertyType ptype = 
                    (DTSPropertyType)attr.getAttribute().getObject();
                props = con.getFetchedProperties();
                //now find the property
                for (i=0; i<props.length; i++)
                    if (props[i].getPropertyType().equals(ptype)) break;
                //DTSProperty prop = new DTSProperty(ptype, (String)val);
                if (i==props.length) {  
                    //property not found (oldVal==""), must have a non-null new value
                    queryMgr.getSearchQuery().addProperty(con, 
                        new DTSProperty(ptype, (String)val));
                }
                else {  
                    //exists, update (or delete) this property
                    if (val.equals("")) {
                        //need to delete
                        queryMgr.getSearchQuery().deleteProperty(con, props[i]);
                    }
                    else {
                        //do an update
                        queryMgr.getSearchQuery().
                            updateProperty(con, props[i], 
                                           new DTSProperty(ptype, (String)val));
                    }
                }
            } catch (Exception e) {
                moduleMgr.showErrorMessage("Error updating property value.");
                Category.getInstance("tableviewer").error(e);
                e.printStackTrace();
                success = false;
            }
        }
        else if ((attr.getAttribute().getObject() instanceof AssociationType) &&
                 attr.getDisplayAttribute().getName().equals(
                  TableViewerAttribute.CONCEPT_NAME)) {
            AssociationType atype = (AssociationType)attr.getAttribute().getObject();
            assns = con.getFetchedConceptAssociations();
            for (i=0; i<assns.length; i++)
                if (assns[i].getAssociationType().equals(atype)) break;
            try {
                //get new target Concept
                DTSConcept toCon = (DTSConcept)QualifiedObject.getObjectForName
                    ((String)val,ConceptAttributeSetDescriptor.ALL_ATTRIBUTES);
                //reject bad (non-null) val
                if (toCon==null && !val.equals("")) { 
                    moduleMgr.showErrorMessage("Unknown Concept: '"+val+"'");
                    return; //will repaint original value
                }
                //build a new Association
                if (i==assns.length) {  
                    //association not found, add the new one
                    queryMgr.getAssociationQuery().addConceptAssociation
                        (new ConceptAssociation(con, atype, toCon));
                }
                else {  
                    //exists, update (or delete) this association
                    if (val.equals(""))
                        queryMgr.getAssociationQuery().
                            deleteConceptAssociation(assns[i]);
                    else {
                        queryMgr.getAssociationQuery().
                            updateConceptAssociation(assns[i], 
                               new ConceptAssociation(con, atype, toCon));
                    }       
                }
                update = true;      //may need to redo row due to dependencies
            } catch (Exception e) {
                moduleMgr.showErrorMessage("Error updating association value.");
                Category.getInstance("tableviewer").error(e);
                e.printStackTrace();
                success = false;
            }
        }
        else {
            System.out.println("Error setting table value at "+row+","+col+": "+val);
            return;
        }
        //finally do the table update if successful
        if (success) {
            super.setValueAt(val, row, col);
            //alternate to updating con via addproperty/addAssociation
            try {
                con = queryMgr.getSearchQuery().
                    findConceptByName(con.getName(), con.getNamespaceId(), 
                                      ConceptAttributeSetDescriptor.ALL_ATTRIBUTES);
            } catch (Exception e) {
                e.printStackTrace();
            }
            super.setValueAt(con, row, 0);
            if (update) updateRow(row);     //may need to recompute dependent cell
        }
    }

    /** update a row */
    public void updateRow(int row) {
        DTSConcept con = (DTSConcept)getValueAt(row,0);
        Vector<Object> data = buildRow(con);
        //writeVec(data);               //test
        dataVector.set(row, data);
        fireTableRowsUpdated(row, row);
    }
    
    /** update a row with a new (revised) concept, concept must have been refreshed */
    public void updateRow(int row, DTSConcept con) {
		try {
			con = DTSAppManager.getQuery().getSearchQuery().
        			findConceptByName(con.getName(), con.getNamespaceId(), 
                          ConceptAttributeSetDescriptor.ALL_ATTRIBUTES);
	        Vector<Object> data = buildRow(con);
	        //writeVec(data);               //test
	        dataVector.set(row, data);
	        fireTableRowsUpdated(row, row);
		} catch (Exception e) {
			e.printStackTrace();
		}
    }
         
    public void addRow(DTSConcept con) {
        Vector<Object> data = buildRow(con);
        addRow(data);
    }

    /** build a row's worth of data  */
    private Vector<Object> buildRow(DTSConcept con) {
        //System.out.println("building row for '"+con.getName()+"'");
        Vector<Object> res = new Vector<Object>();   //row vector
        res.add(con);                //first entry is concept
        for (int i=1, n=getColumnCount(); i<n; i++)     //skip concept
            res.add(getColumnIdentifier(i).getValue(con));
        return res;
    }

    /** for testing */
    private void writeVec(Vector<Object> vec) {
        for (int i=1; i<vec.size(); i++)
            System.out.print("|"+vec.get(i));
        System.out.println();
    }
}