package com.apelon.modules.dts.editor.tableviewer;


import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.EventListener;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.SwingConstants;
import javax.swing.SwingUtilities;
import javax.swing.event.EventListenerList;


/**
 * panel to accept new concept names
 * <p>
 * Copyright (c) 2006 Apelon, Inc. All rights reserved.
 * @since 3.4.0
 */
class TableViewerNamespaceNewPanel extends JPanel implements ActionListener
{
                
    JButton okButton, cancelButton;
    JTextField conceptField;
    String concept = null;          //result
        
    TableViewerNamespaceNewPanel() 
    {
        super();
        //do layout
        GridBagLayout gb = new GridBagLayout();
        setLayout(gb);
        GridBagConstraints gc = new GridBagConstraints();
        gc.anchor = GridBagConstraints.CENTER;
        gc.gridwidth = 1;
        gc.gridheight = 1;
                
        //first the label
        gc.gridx = 0;
        gc.gridy = 0;
        gc.weightx = 0.0;
        gc.weighty = 0.0;
        gc.fill = GridBagConstraints.HORIZONTAL;
        gc.insets = new Insets(10, 15, 5, 5);   //top, left, bottom, right
        JLabel label = new JLabel(" New Concept Name: ", SwingConstants.RIGHT);
        add(label, gc);
                
        gc.gridx++;
        gc.weightx = 1.0;
        gc.insets = new Insets(10, 5, 5, 15);   //top, left, bottom, right
        conceptField = new JTextField(50);
        add(conceptField, gc);
                
        gc.gridy++;
        gc.gridx = 0;
        gc.gridwidth = 2;
        gc.insets = new Insets(5, 15, 10, 15);  //top, left, bottom, right
        JPanel buttonPanel = new JPanel();
        okButton = new JButton(" OK ");
        okButton.addActionListener(this);
        buttonPanel.add(okButton);
        cancelButton = new JButton(" Cancel ");
        cancelButton.addActionListener(this);
        buttonPanel.add(cancelButton);
        add(buttonPanel, gc);

        conceptField.requestFocus();
        //now make big enough for long names
        setPreferredSize(new Dimension(400, 0));
    }
          

    //button event handler
          
    private final static Cursor busyCursor = new Cursor(Cursor.WAIT_CURSOR);
    private final static Cursor defCursor = new Cursor(Cursor.DEFAULT_CURSOR);

    public void actionPerformed(ActionEvent ae)
    {
        if (ae.getSource()==okButton)
        {
            String name = conceptField.getText();
            if (!name.equals(""))
            {
                setCursor(busyCursor);
                fireActionEvent(new ActionEvent(this, 0, name));
                setCursor(defCursor);
                conceptField.requestFocus();
                conceptField.setText("");
            }
        }
        else                                    //cancel
        {
            SwingUtilities.getWindowAncestor(this).dispose();
        }
    }
          

    //action listeners
    EventListenerList listenerList = new EventListenerList();
         
    public void addActionListener(ActionListener al)
    {
        listenerList.add(ActionListener.class, al);
    }
         
    public void removeActionListener(ActionListener al)
    {
        listenerList.remove(ActionListener.class, al);
    }
         
    protected void fireActionEvent(ActionEvent ae)
    {
        EventListener[] listeners = listenerList.getListeners(ActionListener.class);
        for (int i=listeners.length-1; i>=0; i--)
            ((ActionListener)listeners[i]).actionPerformed(ae);
    }
}