package com.apelon.modules.dts.editor.tableviewer;


import java.awt.dnd.DropTargetDropEvent;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Comparator;
import javax.swing.JDialog;
import javax.swing.JOptionPane;

import com.apelon.apps.dts.editor.modules.DTSEditorModuleMgr;
import com.apelon.dts.client.association.AssociationType;
import com.apelon.dts.client.concept.ConceptAttributeSetDescriptor;
import com.apelon.dts.client.concept.DTSConcept;
import com.apelon.dts.client.concept.DTSSearchOptions;
import com.apelon.dts.client.concept.OntylogConcept;
import com.apelon.dts.client.events.DataChangeEvent;
import com.apelon.dts.client.events.NamespaceEvent;
import com.apelon.dts.client.namespace.Namespace;
import com.apelon.dts.client.namespace.NamespaceType;
import com.apelon.dts.dtsutil.QualifiedObject;
import org.apache.log4j.Category;
 

/**
 * selection panel for namespace lists
 * <p>
 * Copyright (c) 2006 Apelon, Inc. All rights reserved.
 * @since 3.4.0
 */
class TableViewerNamespacePanel extends TableViewerDirectorPanel implements TableViewerTableDirector {
	
    static final String THESAURUS_AXIS = "Parent Of";	//name of default thesaurus association    
    final static ConceptAttributeSetDescriptor allCASD = ConceptAttributeSetDescriptor.ALL_ATTRIBUTES;

    AssociationType axisType;					//association Type for Thesaurus axis
    DTSSearchOptions sopts = 
        new DTSSearchOptions(9999, DTSSearchOptions.ALL_NAMESPACES, allCASD);
                
    TableViewerNamespacePanel(DTSEditorModuleMgr mgr, TableViewerTablePanel panel) {
        super(mgr, panel, LOAD+OPEN);
    }
         
    public int getActiveNamespaceId() {
        Namespace space = getNamespace(getActiveName());
        if (space!=null) return space.getId();
        return -1;
    }
         
    /**************************************************/

    /** get the label for this panel */
    public String getDirectorLabel() { return "Namespace"; }
    
    /** get sorted list of namespace names */
    protected String[] getDirectorNames() {
        String[] names = null;
        try {
            Namespace[] spaces = queryMgr.getNamespaceQuery().getNamespaces();
            names = new String[spaces.length];
            for (int i=0; i<spaces.length; i++) names[i] = spaces[i].getName();
            Arrays.sort(names, new Comparator<String>() { 
                public int compare(String a, String b) {
                    return a.compareToIgnoreCase(b);
                }
            });
        } catch (Exception e) {
            Category.getInstance("tableviewer").error(e);
            e.printStackTrace();
        }
        return names;
    }

    //methods for Namespace Lists

    private String getKey(String name) {
        return "["+name+"]";
    }
         
    /** load table, 
     * call with element name, load function (LOAD/OPEN)
     * can't use active variables */
    protected boolean loadTable(String name, int func) {
    	boolean done = false;
        DTSConcept[] cons = new DTSConcept[0];
        Namespace namespace = getNamespace(name);
        if (namespace==null) {
            moduleMgr.showErrorMessage("Error accessing table namespace.");
        	return false;
        }
        setCursor(busyCursor);
        try {
        	//set up Thesaurus axis
    		if (namespace.getNamespaceType()==NamespaceType.ONTYLOG || 
    				namespace.getNamespaceType()==NamespaceType.ONTYLOG_EXTENSION)
    			axisType = null;
    		else {
    			axisType = queryMgr.getAssociationQuery().
    				findAssociationTypeByName(THESAURUS_AXIS, namespace.getId());
    		}
        	
            //get all concepts in selected namespace (CASD is ALL) 
            // if getConcept is true

            if (func==LOAD) {
                sopts.setNamespaceId(namespace.getId());
                System.out.println("TableViewer.getting concepts for Namespace '"+
                                   namespace.getName()+"'");
                cons = queryMgr.getSearchQuery().findConceptsWithNameMatching("*", 
                                                                              sopts);
                System.out.println("TableViewer.found "+cons.length+" concepts");
            }
            else {
                System.out.println("TableViewer.opening table for Namespace '"+
                                   namespace.getName()+"'");
            }
            tablePanel.loadTableViewer(this, getKey(name), cons, 
                 (namespace.isLocal()?TableViewerTablePanel.NEW_DELETE_CONCEPTS:0) |
                                       TableViewerTablePanel.ADD_REMOVE_ROWS);
            done = true;
        } catch (Exception e) {
            Category.getInstance("tableviewer").error(e);
            System.out.println("TableViewer.error loading concepts!");
            e.printStackTrace();
            moduleMgr.showErrorMessage("Error retrieving Namespace concepts.");
        }
        setCursor(defCursor);
        return done;
    }
         
    private Namespace getNamespace(String name) {
        try {
            return queryMgr.getNamespaceQuery().findNamespaceByName(name);
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }
    }
    
    /**************************************************/

    /** add additional rows to table */
    public void addRows(Object obj) {
        final TableViewerConceptAddPanel panel = 
            new TableViewerConceptAddPanel((DropTargetDropEvent)obj);
        panel.addActionListener( new ActionListener() {
                public void actionPerformed(ActionEvent ae) {
                    addRows(panel.getConcepts(), ae.getID());
                }
        }
                                 );
        JDialog dialog = moduleMgr.createDialog(panel, "Add Concepts", 400, 150);
        dialog.setModal(false);
        dialog.setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
        dialog.setVisible(true);
    }
         
    /** add/remove rows from table, invoked from concept add event */
    public void addRows(DTSConcept[] concepts, int code) {
        DTSConcept con, cons[];
        String qname;
                
        //false=add, true=delete
        boolean remfunc = code/TableViewerConceptAddPanel.DELETE>0; 
        code = code%TableViewerConceptAddPanel.DELETE;          //action code
        int currentId = getActiveNamespaceId();
        if (currentId<0) {
            moduleMgr.showErrorMessage("Error accessing table namespace.");
        	return;
        }
    
        //process the concepts
        for (int i=0; i<concepts.length; i++) {
            if ((con=concepts[i])==null) continue;
            boolean ontylog = isOntylogConcept(con);
            qname = QualifiedObject.getNameForObject(con);
            if (code%2==0) {         //do we add base concept?
                try {   //fill out the CASD
                	cons = new DTSConcept[1];
                	cons[0] = con;
                	cons = fillConceptsWithAttributes(cons, allCASD);
                } catch (Exception e) {
                    Category.getInstance("tableviewer").error(e);
                    System.out.println("TableViewer.error fetching data for: "+qname);
                    e.printStackTrace();
                    moduleMgr.showErrorMessage("Error fetching data for '"+
                                               qname+"'.");
                    cons = new DTSConcept[0];
                }
                if (remfunc) remEntries(cons);
                else addEntries(cons, currentId);
            }
            if (code==TableViewerConceptAddPanel.CHILDREN || 
                code==TableViewerConceptAddPanel.CONCEPT_AND_CHILDREN) {
                try  {
                    //** don't know why separate array needed here and 
                        //not in descendants **)
                	if (ontylog) cons = queryMgr.getNavQuery().
                        getNavChildContext((OntylogConcept)con, allCASD).getChildren();
                	else cons = queryMgr.getNavQuery().
                		getNavChildContext(con, allCASD, axisType).getChildren();
                    //have to get attributes for children
                    cons = fillConceptsWithAttributes(cons, allCASD);
                    if (remfunc) remEntries(cons);
                    else addEntries(cons, currentId);
                } 
                catch (Exception e) 
                {
                    Category.getInstance("tableviewer").error(e);
                    System.out.println("TableViewer.error fetching children for: "+qname);
                    e.printStackTrace();
                    moduleMgr.showErrorMessage("Error fetching children for '"+qname+"'.");
                    cons = new DTSConcept[0];
                }
            }
            else if (code==TableViewerConceptAddPanel.DESCENDANTS || 
                     code==TableViewerConceptAddPanel.CONCEPT_AND_DESCENDANTS) {
                try  {
                    if (ontylog) cons = queryMgr.getClassQuery().
                        getSubConcepts((OntylogConcept)con, allCASD);
                    else cons = getThesaurusDescendants(con, allCASD);
                    cons = fillConceptsWithAttributes(cons, allCASD);
                    if (remfunc) remEntries(cons);
                    else addEntries(cons, currentId);
                } 
                catch (Exception e) {
                    Category.getInstance("tableviewer").error(e);
                    System.out.println("TableViewer.error fetching descendants for: "+qname);
                    e.printStackTrace();
                    moduleMgr.showErrorMessage("Error fetching descendants for '"+qname+"'.");
                }
            }
        }
    }
    
    /** is a concept an Ontylog concept? 
     * because all DnD concepts are OntylogConcepts, tests the namespace */
    private boolean isOntylogConcept(DTSConcept con) {
    	try {
    		NamespaceType type =  queryMgr.getNamespaceQuery().
    			findNamespaceById(con.getNamespaceId()).getNamespaceType();
    		return type==NamespaceType.ONTYLOG || type==NamespaceType.ONTYLOG_EXTENSION;
    	}catch (Exception e) {
            Category.getInstance("tableviewer").error(e);
            System.out.println("TableViewer.error fetching namespace for '"+con+"'");
            e.printStackTrace();
            moduleMgr.showErrorMessage("Error fetching namespace for '"+con+"'.");
    		return false;
    	}
    }

    /**
     * get Thesaurus descendants for a concept,
     */
    private DTSConcept[] getThesaurusDescendants(DTSConcept con, 
                     ConceptAttributeSetDescriptor casd) throws Exception {
        ArrayList<DTSConcept> desc = new ArrayList<DTSConcept>();       //this is faster than array sloughing
    	DTSConcept[] kids = queryMgr.getNavQuery().
    				getNavChildContext(con, casd, axisType).getChildren();
    	fillConceptsWithAttributes(kids, casd);
        int i, j, len = kids.length;
        for (i=0; i<len; i++) desc.add(kids[i]);
        for (i=0; i<len; i++){
            kids = getThesaurusDescendants((DTSConcept)desc.get(i), casd);
            //add kids to desc
            for (j=0; j<kids.length; j++) desc.add(kids[j]);
        }
        DTSConcept[] res = new DTSConcept[0];
        return (DTSConcept[])desc.toArray(res);
    }
        
    /**
     * fill an array of concepts with attributes specified in casd
     * used for parents and ancestors, use new array since input may be casted
     * and findConcept... returns OntylogConcept[]
     */
    private DTSConcept[] fillConceptsWithAttributes(DTSConcept[] cons, 
            								ConceptAttributeSetDescriptor casd) throws Exception {
    	DTSConcept[] res = new DTSConcept[cons.length];
   		for (int i=0; i<cons.length; i++) {
    		res[i] = queryMgr.getSearchQuery().
    			findConceptById(cons[i].getId(), cons[i].getNamespaceId(), casd);
   		}
   		return res;
    }

    /*******************************************************************************/
    /** add new rows for concepts in cons */
    private void addEntries(DTSConcept[] cons, int id) {
        String qname;
        for (int i=0; i<cons.length; i++) {
            qname = QualifiedObject.getNameForObject(cons[i]);
            if (cons[i].getNamespaceId()!=id) {
                System.out.println("TableViewer.'"+qname+"' not in active Namespace.");
                moduleMgr.showErrorMessage("'"+qname+"' not in active Namespace.");
                continue;
            }
            if (tablePanel.isRowPresent(qname)) {
                System.out.println("TableViewer.'"+qname+"' already present.");
                moduleMgr.showErrorMessage("'"+qname+"' already present.");
                continue;
            }
            //finally add it
            tablePanel.addRow(cons[i]);
        }
    }
         
    /** remove rows for concepts in cons; don't worry about confirm */
    private void remEntries(DTSConcept[] cons) {
        String qname;
        int row;
        for (int i=0; i<cons.length; i++) {
            qname = QualifiedObject.getNameForObject(cons[i]);
            if ((row=tablePanel.findRowForConcept(qname))<0) {
                System.out.println("TableViewer.entry for '"+qname+"' not present.");
                continue;
            }
            //now remove the row (must already exist)     
            tablePanel.removeRow(row);
        }
    }
         
    /** add new concept to namespace and table, results returned via event */
    public void newConcept() {
        TableViewerNamespaceNewPanel panel = new TableViewerNamespaceNewPanel();
        panel.addActionListener( new ActionListener() {
                public void actionPerformed(ActionEvent ae) {
                    newRow(ae.getActionCommand());
                }
        }
                                 );
        //moduleMgr.showFloatPanel(panel, "New Namespace Concept", 400, 150);
        JDialog dialog = moduleMgr.createDialog(panel, 
                                                "New Namespace Concept", 400, 150);
        dialog.setModal(false);
        dialog.setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
        dialog.setVisible(true);
    }
         
    /** add a new Namespace row (concept), from dialog event */
    public void newRow(String name) {
        DTSConcept con;
                
        //need qualified name
        String space = getActiveName();
        String qname = name+"["+space+"]";
        System.out.println("TableViewer.creating new concept: '"+qname+"'");
        if (tablePanel.isRowPresent(qname)) {
            System.out.println("TableViewer.concept for '"+qname+"' already present.");
            moduleMgr.showErrorMessage("Concept for '"+name+"' already present.");
            //**moduleMgr.showWarningMessage("Concept for '"+name+"' already present.");
            return;
        }
                
        try {
            int id = getActiveNamespaceId();
            if (id<0) {
                moduleMgr.showErrorMessage("Error accessing table namespace.");
            	return;
            }
            con = new DTSConcept(name, -1, null, id);
            con = queryMgr.getThesaurusQuery().addConcept(con);
            tablePanel.addRow(con);
        } catch (Exception e) {
            Category.getInstance("tableviewer").error(e);
            e.printStackTrace();
            System.out.println("TableViewer.error creating concept: "+name);
            moduleMgr.showErrorMessage("Error creating concept: "+name);
        }
    }

         
    /** remove namespace rows from table */
    public void removeRows(int[] rows) {
        int res, row;
        boolean doall = false;
        Object[] options = { "Yes", "Yes to all", "No" };
        for (int n=rows.length-1; n>=0; n--) {  //do backwards so removeRow works
            row = rows[n];
            DTSConcept con = tablePanel.getConceptForRow(row);
            if (!doall) {
                res = JOptionPane.showOptionDialog(null, 
                   "Are you sure you want to remove concept:\n'"+con.getName()+"'?",
                                                   "Confirm Remove", 
                                                   JOptionPane.DEFAULT_OPTION, 
                                                   JOptionPane.WARNING_MESSAGE, 
                                                   null, options, options[0]);
                if (res==2) continue;       //skip this one
                if (res==1) doall = true;
                else if (res!=0) break;     //cancelled out
            }               
            tablePanel.removeRow(row);      //just remove from the display
        }
    }
         
    /** delete namespace row concepts */
    public void deleteConcepts(int[] rows) 
    {
        int res, row;
        boolean doall = false;
        Object[] options = { "Yes", "Yes to all", "No" };
        for (int n=rows.length-1; n>=0; n--) {  //do backwards so removeRow works
            row = rows[n];
            DTSConcept con = tablePanel.getConceptForRow(row);
            if (!doall) {
                res = JOptionPane.showOptionDialog(null, 
                   "Are you sure you want to delete concept:\n'"+con.getName()+"'?",
                                                   "Confirm Delete", 
                                                   JOptionPane.DEFAULT_OPTION, 
                                                   JOptionPane.WARNING_MESSAGE, 
                                                   null, options, options[0]);
                if (res==2) continue;     //skip this one
                if (res==1) doall = true;
                else if (res!=0) break;   //cancelled out
            }               
            System.out.println("TableViewer.deleting concept'"+con.getName()+"'");
            try {
                queryMgr.getThesaurusQuery().deleteConcept(con);
                tablePanel.removeRow(row);
            } catch (Exception e) {
                Category.getInstance("tableviewer").error(e);
                e.printStackTrace();
                System.out.println("TableViewer.error deleting concept: "+con.getName());
                moduleMgr.showErrorMessage("Error deleting concept: "+con.getName());
            }
        }
    }
    
    //* clear the table */
    public void clear() {
	 	Object[] options = { "Yes", "No" };
 		int res = JOptionPane.showOptionDialog(null, 
	 		"Are you sure you want to remove all concepts\nand reset the table format?",
	 			"Confirm Clear", 
				JOptionPane.YES_NO_OPTION,
				JOptionPane.WARNING_MESSAGE, 
				null, options, options[0]);
	 	if (res==0) tablePanel.clearTableData();
    }
    
    /*******************************************************************************/
    //handle namespace event
    void processNamespaceEvent(NamespaceEvent event) {
        //System.out.println("namespace action:"+event);
    	if (event.getEventType()==DataChangeEvent.EVENT_TYPE_NEW) {
    		loadKeyCombo("", "");				//just load the new name
    	}
        if (event.getEventType() == NamespaceEvent.EVENT_TYPE_DELETE) {
        	loadKeyCombo(event.getOldNamespace().getName(), "");
        }
        if (event.getEventType()==NamespaceEvent.EVENT_TYPE_MODIFY) {
        	loadKeyCombo(event.getOldNamespace().getName(),event.getNewNamespace().getName());
        }
    }
}