package com.apelon.modules.dts.editor.tableviewer;


import java.awt.BorderLayout;

import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.JSplitPane;
import javax.swing.JTabbedPane;

import com.apelon.apps.dts.editor.modules.DTSEditorModuleMgr;
import com.apelon.apps.dts.editor.modules.DTSModuleConfig;
import com.apelon.beans.dts.plugin.DTSAppManager;
import com.apelon.beans.dts.plugin.connection.ConnectionCloseVeto;
import com.apelon.beans.dts.plugin.connection.DtsConnectionEvent;
import com.apelon.beans.dts.plugin.connection.DtsConnectionListener;
import com.apelon.dts.client.events.ConceptEvent;
import com.apelon.dts.client.events.ConceptListener;
import com.apelon.dts.client.events.NamespaceEvent;
import com.apelon.dts.client.events.NamespaceListener;
import com.apelon.dts.client.events.SubsetEvent;
import com.apelon.dts.client.events.SubsetListener;

        
/**
 * manage concept review via a table
 * <p>
 * Copyright (c) 2006 Apelon, Inc. All rights reserved.
 * @since 3.4.0
 */
public class TableViewerPanel extends JPanel 
		implements DtsConnectionListener, ConceptListener, NamespaceListener, SubsetListener {
	
    TableViewerTablePanel tablePanel;
    TableViewerSubsetPanel subsetPanel;
    TableViewerNamespacePanel spacePanel;
    DTSEditorModuleMgr moduleMgr;
        
    public TableViewerPanel(DTSEditorModuleMgr mgr, DTSModuleConfig cfg) {
        super(new BorderLayout());
        moduleMgr = mgr;
        JSplitPane splitPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT, true);
        splitPane.setOneTouchExpandable(true);
        splitPane.setDividerSize(10);

        //get table panel first because we need table in namespace 
        //and subset panels

        splitPane.setRightComponent(buildTablePanel(mgr, cfg));
        splitPane.setLeftComponent(buildSourcePanel(mgr));
        add(splitPane, "Center");
        mgr.registerConnectionListener(this);
        enableModule(mgr.getServerConnection()!=null);
    }
          
    /** enable the modules */
    public void enableModule(boolean enable) {
        spacePanel.enableModule(enable);
        subsetPanel.enableModule(enable);
        tablePanel.enableModule(enable);
        if (enable) {
        	try {
                DTSAppManager.getQuery().getAssociationQuery().addConceptListener(this);
                DTSAppManager.getQuery().getDTSConceptQuery().addConceptListener(this);
                DTSAppManager.getQuery().getThesaurusQuery().addConceptListener(this);
                DTSAppManager.getQuery().getNamespaceQuery().addNamespaceListener(this);
                DTSAppManager.getQuery().getSubsetQuery().addSubsetListener(this);
                } catch (Exception e) {
                	e.printStackTrace();
                }
        	
        } else {
            try {
                DTSAppManager.getQuery().getAssociationQuery().removeConceptListener(this);
                DTSAppManager.getQuery().getDTSConceptQuery().removeConceptListener(this);
                DTSAppManager.getQuery().getThesaurusQuery().removeConceptListener(this);
                DTSAppManager.getQuery().getNamespaceQuery().removeNamespaceListener(this);
                DTSAppManager.getQuery().getSubsetQuery().removeSubsetListener(this);
                } catch (Exception e) {
                	//e.printStackTrace();	//likely thrown because there is no connection
                }
        }
    }
          
    //DtsConnectionListener interface for floating panels
                
    public void connectionOpened(DtsConnectionEvent event){
        System.out.println ("TableViewerPanel.connectionOpened(): event received");
        enableModule(true);
    }

    public void connectionClosed(DtsConnectionEvent event){
        System.out.println ("TableViewerPanel.connectionClosed(): event received");
        enableModule(false);
    }

    public void connectionClosing(DtsConnectionEvent event) { }

    public void connectionWillClose(DtsConnectionEvent event, 
                                    ConnectionCloseVeto veto) { }
                
    /**************************************************************************/

    private JComponent buildSourcePanel(DTSEditorModuleMgr manager) {
        JTabbedPane tabPanel = new JTabbedPane();
        spacePanel = new TableViewerNamespacePanel(manager, tablePanel);
        tabPanel.add("View "+spacePanel.getDirectorLabel(), spacePanel);
        subsetPanel = new TableViewerSubsetPanel(manager, tablePanel);
        tabPanel.add("View "+subsetPanel.getDirectorLabel(), subsetPanel);
        return tabPanel;
    }
          
    private TableViewerTablePanel buildTablePanel(DTSEditorModuleMgr manager, 
    								DTSModuleConfig config) {
        tablePanel = new TableViewerTablePanel(manager, config);
        return tablePanel;
    }
    
    /**************************************************************************/
    //DTS event listeners
    public void conceptActionOccurred(ConceptEvent event) {
        //System.out.println("concept action:"+event);
    	if (!tablePanel.isLoaded()) return;					//must be active 
        //displayConceptEvent(event);
    	int n = tablePanel.processConceptEvent(event);
    	if (n>0) System.out.println("TableViewer: "+n+" row"+(n>1?"s":"")+" updated on '"+event.getConcept().getName()+"' event.");
    	if (n<0) System.out.println("TableViewer: "+n+" row"+(n>1?"s":"")+" deleted on '"+event.getConcept().getName()+"' event.");
    }

    public void namespaceActionOccurred(NamespaceEvent event) {
        //System.out.println("Namespace action:"+event);
    	spacePanel.processNamespaceEvent(event);				//pass through always
    }

    public void subsetActionOccurred(SubsetEvent event) {
        //System.out.println("Subset action:"+event);
    	subsetPanel.processSubsetEvent(event);
    }

    /*
    private void displayConceptEvent(ConceptEvent event) {
    	ConceptAttributeChange change = event.getAttributeChange();
    	System.out.println("Concept Event on '"+event.getConcept().getName()+
    			"': "+getEventTypeName(event.getEventType())+
    			", "+getChangeAttributeName(change));
    }
    
    private String getEventTypeName(int type) {
    	switch (type) {
    		case DataChangeEvent.EVENT_TYPE_DELETE: return "delete";
    		case DataChangeEvent.EVENT_TYPE_NEW: return "new";
    		case DataChangeEvent.EVENT_TYPE_MODIFY: return "modify";
    		default: return "unknown";
    	}
    }
    
    private String getChangeAttributeName(ConceptAttributeChange change) {
    	String attr, type;
    	if (change==null) return "null";
    	switch (change.getChangeType()) {
			case ConceptAttributeChange.DELETE:	type = "delete";
												break;
			case ConceptAttributeChange.NEW:	type = "new";
												break;
			case ConceptAttributeChange.MODIFY:	type = "modify";
												break;
			default:	type = "unknown";
		}
    	switch (change.getAttributeType()) {
    		case ConceptAttributeChange.ASSOCIATION:	attr = "association";
														break;
    		case ConceptAttributeChange.PROPERTY:	attr = "property";
													break;
    		case ConceptAttributeChange.NAME:	attr = "name";
												break;
    		case ConceptAttributeChange.ROLE:	attr = "role";
												break;
    		case ConceptAttributeChange.DEFININGCONCEPT:	attr = "defining concept";
															break;
    		case ConceptAttributeChange.PRIMITIVE:	attr = "primitive";
													break;
    		case ConceptAttributeChange.ROLEGROUP:	attr = "role group";
													break;
    		default:	attr = "unknown";
    	}
    	return attr+"|"+type;
    }
	*/
    
    
          
}