package com.apelon.modules.dts.editor.tableviewer;


import java.awt.Color;
import java.awt.Component;
import java.awt.Point;
import java.awt.dnd.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.Vector;
import javax.swing.DefaultCellEditor;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.SwingUtilities;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.DefaultTableColumnModel;
import javax.swing.table.JTableHeader;
import javax.swing.table.TableColumn;
import javax.swing.table.TableColumnModel;

import com.apelon.apps.dts.editor.modules.DTSEditorModuleMgr;
import com.apelon.beans.dts.dnd.DtsDnd;
import com.apelon.beans.dts.plugin.DTSAppManager;
import com.apelon.beans.dts.plugin.transferable.ConceptTransferable;
import com.apelon.dts.client.DTSQuery;
import com.apelon.dts.client.concept.DTSConcept;
import com.apelon.dts.dtsutil.QualifiedObject;

        
/**
 * JTable with support for adding and sorting columns
 * Copyright (c) 2006 Apelon, Inc. All rights reserved.
 * @since 3.4.0
 */
class TableViewerTable extends JTable 
    implements DropTargetListener, DragGestureListener, DragSourceListener {
    JPopupMenu popupMenu;
    JMenuItem addCol, remCol;
    TableViewerModel tableModel;
    TableViewerSorter sortModel;
    DTSEditorModuleMgr moduleMgr; 
    TableViewerTablePanel parent;			//main panel component
    DTSQuery queryMgr;
        
    TableViewerTable(DTSEditorModuleMgr mgr, TableViewerTablePanel panel)
    {
        super();
        moduleMgr = mgr;
        parent = panel;
        queryMgr = DTSAppManager.getQuery();
        setAutoCreateColumnsFromModel(false);   //we'll handle the columns
        setDefaultRenderer(Object.class, new CodeCellRenderer());
        DefaultCellEditor editor = new DefaultCellEditor(new JTextField());
        editor.setClickCountToStart(1);
        setDefaultEditor(String.class, editor);
        //need to do these manually now
        setColumnModel(new DefaultTableColumnModel());
        TableViewerTableHeader header = new TableViewerTableHeader(getColumnModel());
        setTableHeader(header);
        header.addMouseListener(new MouseHandler());
        tableModel = new TableViewerModel(moduleMgr);
        sortModel = new TableViewerSorter(tableModel);
        setModel(sortModel);
        sortModel.setTableHeader(header);
        setColumnSelectionAllowed(true);
        setRowSelectionAllowed(true);

        popupMenu = new JPopupMenu();
        addCol = new JMenuItem("Add Column");
        addCol.addActionListener( new ActionListener()
        {
            public void actionPerformed(ActionEvent ae)
            {
                addColumn();
            }
        }
                                  );
        popupMenu.add(addCol);
        remCol = new JMenuItem("Delete Column");
        remCol.addActionListener( new ActionListener()
        {
            public void actionPerformed(ActionEvent ae)
            {
                removeColumn();
            }
        }
                                  );
        popupMenu.add(remCol);
                
        dropTarget = new DropTarget(this, DnDConstants.ACTION_MOVE, this, true);
        dragSource = new DragSource();
        recognizer = dragSource.createDefaultDragGestureRecognizer
            (this, DnDConstants.ACTION_MOVE, this);
    }
        
    TableViewerSorter getTableModel() 
    {
        return sortModel;
    }
        
    /**************************************************/

    /** custom table header for separate tool tips */
    class TableViewerTableHeader extends JTableHeader
    {
        
        public TableViewerTableHeader(TableColumnModel cm)
        {
            super(cm);
        }
                
        /** Implement table header tool tips.*/
        public String getToolTipText(MouseEvent me) 
        {
            int index = columnModel.getColumnIndexAtX(me.getX());
            return index<0?null:
                (String)columnModel.getColumn(index).getHeaderValue();
        }
    }   
    
    
    /**************************************************/

    /**
     * cell renderer to show DnD target
     * note that row, column are view indices not model indices
     */
    class CodeCellRenderer extends DefaultTableCellRenderer 
    { 
        public Component getTableCellRendererComponent(JTable table, 
                                                       Object value,
                                                       boolean isSelected, 
                                                       boolean hasFocus,
                                                       int row, 
                                                       int column) 
        {
            Component c = super.getTableCellRendererComponent(table,
                                                              value,
                                                              isSelected,
                                                              hasFocus,
                                                              row,
                                                              column);
            setOpaque(true);
            if (hoverCell.y>=0 && hoverCell.x==column && hoverCell.y==row)
            {
                c.setBackground(Color.yellow);
            }
            else
                c.setBackground(isSelected?
                                TableViewerTable.this.selectionBackground:
                                TableViewerTable.this.getBackground());       
            return c;
        }
    }


    int popupColumn;    //columnIndex for column selected during popup operation
        
    private void addColumn() {
        int n, col;
        TableColumn column;
        
        TableViewerAttribute attr = 
              TableViewerAttrDialog.showDialog(this.getParent(), parent.getDirector());
        if (attr==null) return;                //nothing selected
        String name = attr.toString();
        //System.out.println("selected: "+name);
        TableColumnModel colModel = getColumnModel();
        //see if column already exists
        for (col=0, n=colModel.getColumnCount(); col<n; col++)
            if (name.equals((String)colModel.getColumn(col).getHeaderValue())) break;
                        
        if (col==n) {                                   //new column
            Vector<String> data = buildColumn(attr);    //make a data vector for column
            //System.out.println("New column="+col);    //new modelIndex
            tableModel.addColumn(attr, data); 			//add the column to the (end of the) data model
        }
        else {       //column already exists in data model
            moduleMgr.showErrorMessage("Attribute column already present.");
            return;         
        }
        //create a new table column, column data is last in the model (col+1)
        column = new TableColumn(tableModel.getColumnCount()-1);   
        column.setHeaderValue(attr.toString());
        //now add the column and place it in new location                       
        colModel.addColumn(column);

        //move to desired position; to right of selected column
        colModel.moveColumn(colModel.getColumnCount()-1, popupColumn+1);  
}
        
    private void removeColumn() {
        if (getColumnCount()==1) {
            moduleMgr.showErrorMessage("Cannot delete only column.");
            return;         
        }
        TableColumn column = getColumnModel().getColumn(popupColumn);
        removeColumn(column);
    }
        
    /** build a new column based on the attribute in 'name' */ 
    private Vector<String> buildColumn(TableViewerAttribute attr) {
        Vector<String> res = new Vector<String>();
        for (int i=0, n=getRowCount();i<n; i++) {
            res.add(attr.getValue((DTSConcept)sortModel.getValueAt(i, 0)));
            //System.out.println(i+"="+res.get(i));
        }
        return res;
    }
        
    /**************************************************/

    /** handle click in header, if right button show popup */
    private class MouseHandler extends MouseAdapter {
        public void mouseClicked(MouseEvent me) {
            if (!SwingUtilities.isRightMouseButton(me)) return;
            JTableHeader header = (JTableHeader) me.getSource();
            TableColumnModel columnModel = header.getColumnModel();
            popupColumn = columnModel.getColumnIndexAtX(me.getX());
            remCol.setEnabled(popupColumn>=0);
            //System.out.println("popupColumn="+popupColumn);
            //System.out.println("table column="+columnAtPoint(me.getPoint()));
            if (popupColumn<0) popupColumn = columnModel.getColumnCount();//add at end
            popupMenu.show((Component)me.getSource(), me.getX(), me.getY());
        }
    }
         
    /**************************************************/
    //Drag and drop
         
    DragSource dragSource;
    DragGestureRecognizer recognizer;
    private boolean dragPending = false;           //to prevent self drop
         
    /** DragGestureListener interface */
    public void dragGestureRecognized(DragGestureEvent dge)
    {
        int row = getSelectedRow();
        if (row>=0)
        {
            int col = convertColumnIndexToModel(columnAtPoint(dge.getDragOrigin()));
            if (!sortModel.isCellEditable(row, col))//don't let drag confuse editing
            {
                ConceptTransferable ct = 
                    new ConceptTransferable((DTSConcept)sortModel.getValueAt(row,0));
                dge.startDrag(DragSource.DefaultMoveDrop, ct, this);
                dragPending = true;
            }
        }
    }
         

    //DragSourceListener interface

    public void dragDropEnd(DragSourceDropEvent dse) { dragPending = false; }
    public void dragEnter(DragSourceDragEvent dse) {}
    public void dragExit(DragSourceEvent dse) {}
    public void dragOver(DragSourceDragEvent dse) {}
    public void dropActionChanged(DragSourceDragEvent dse) {}
        
    DropTarget dropTarget;
         
    /** DropTargetListener interface*/
    public void dragEnter(DropTargetDragEvent de)
    {
        de.acceptDrag(DnDConstants.ACTION_MOVE);
    }
         
    public void drop(DropTargetDropEvent de)
    {
        int row, col;
        try
        {
            Vector vec = DtsDnd.checkForMultiDrop(de);
            if (vec==null)
            {       //not a multi
                DTSConcept con = DtsDnd.checkForConcept(de);
                if ( con!=null && isDroppable(de.getLocation()) )
                {       //is a concept!
                    de.acceptDrop(DnDConstants.ACTION_MOVE);
                    col = hoverCell.x;
                    row = hoverCell.y;
                    String s = QualifiedObject.getNameForObject(con);
                    //System.out.println("Valid drop recognized at "+col+","+row+
                    //                   ": "+s);
                    int modcol = convertColumnIndexToModel(col); //need model index
                    sortModel.setValueAt(s, row, modcol); //this will change DTS KB
                    de.dropComplete(true);
                    sortModel.fireTableCellUpdated(row, modcol);
                    hoverCell.setLocation(-1, -1);
                    return;
                }
            }
            de.rejectDrop();
        } catch (Exception e)
        {
            System.out.println(e);
        }
    }
         
    
    /**
     * droppable cell location for use by renderer
     * (col, row) are view indices
     */
    final Point hoverCell = new Point(-1, -1);
       
  
    /**
     * test if droppable condition:
     * table editable, not a self-drop, and point is in a droppable cell
     * if droppable, mark cell for repaint
     */
    private boolean isDroppable(Point pt)
    {
        if (dragPending) return false; //no self drop
        int row, col, modcol;
        col = columnAtPoint(pt);               //may be negative
        row = rowAtPoint(pt);
        //if (col<0 || row<0) System.out.println(pt+":"+row+","+col);
        if (row==hoverCell.y && col==hoverCell.x) return true;   //already there
        if (hoverCell.y>=0) 
        {      //left the hover cell (could be off table)
            //simplification, if new cell is also droppable, 
            // will pick up on next move
            //System.out.println("clearing hover");
            sortModel.fireTableCellUpdated(hoverCell.y, 
                                           convertColumnIndexToModel(hoverCell.x));
            hoverCell.setLocation(-1,-1);
            return false;
        }
        //over new cell (hoverCell==(-1,-1)), is it droppable?
        modcol = convertColumnIndexToModel(col);               //need model index
        if (modcol>=0)
        {
            TableViewerAttribute attr = 
                tableModel.getColumnIdentifier(modcol);
            if (!attr.isLocal()) return false;
            //must be an indirect NAME attribute
            if (attr.getDisplayAttribute()!=null && 
                attr.getDisplayAttribute().getName().
                equals(TableViewerAttribute.CONCEPT_NAME) )
            {
                //got a droppable cell
                hoverCell.setLocation(col, row);
                //System.out.println("new hover cell: "+col+","+row);
                sortModel.fireTableCellUpdated(row, modcol);           //repaint
                return true;
            }
        }
        return false;
    }
         
    public void dragOver(DropTargetDragEvent de) 
    {
        isDroppable(de.getLocation());
    }
         
    final static Point dummyPoint = new Point(-1, -1);
         
    public void dragExit(DropTargetEvent de) 
    {
        isDroppable(dummyPoint);        //to clear any current hoverCell
    }
    public void dropActionChanged(DropTargetDragEvent de) {}
}
