package com.apelon.modules.dts.editor.tableviewer;


import java.awt.BorderLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.File;
import java.util.Enumeration;
import java.util.StringTokenizer;
import java.util.Vector;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.ListSelectionModel;
import javax.swing.border.EmptyBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.TableColumnModelEvent;
import javax.swing.event.TableColumnModelListener;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.SwingUtilities;
import javax.swing.table.DefaultTableColumnModel;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableColumn;

import com.apelon.apps.dts.editor.modules.DTSEditorModuleMgr;
import com.apelon.apps.dts.editor.modules.DTSModuleConfig;
import com.apelon.beans.dts.plugin.DTSAppManager;
import com.apelon.dts.client.DTSQuery;
import com.apelon.dts.client.concept.ConceptAttributeSetDescriptor;
import com.apelon.dts.client.concept.DTSConcept;
import com.apelon.dts.client.concept.DTSSearchOptions;
import com.apelon.dts.client.events.ConceptEvent;
import com.apelon.dts.client.events.DataChangeEvent;
import com.apelon.dts.dtsutil.QualifiedObject;
import com.apelon.dts.dtsutil.SimpleFileFilter;


/**
 * table panel for Table Viewer
 * Copyright (c) 2008 Apelon, Inc. All rights reserved.
 * @since 3.4.0
 * @author jbowie
 */

public class TableViewerTablePanel extends JPanel 
    implements ActionListener, PropertyChangeListener,
               TableModelListener, TableColumnModelListener {
	
    final static boolean debug = false;      
    final static ConceptAttributeSetDescriptor allCASD = 
        ConceptAttributeSetDescriptor.ALL_ATTRIBUTES;

    //button option constants
    final static int ADD_REMOVE_ROWS = 1;
    final static int NEW_DELETE_CONCEPTS = 2;
    
    //sorting constants
    final static String SORT_ASCENDING = "+";
    final static String SORT_DESCENDING = "-";
    final static String SORT_NONE = "0";

    JLabel title;
    TableViewerConfig config;
    TableViewerTable viewTable;
    TableViewerSorter tableModel;
    int options = 0;   							//editing options, see loadTableViewer()
    TableViewerTableDirector director;                    //current table controller
    
    JButton findButton, clearButton, newButton, remButton, delButton, exportButton;
    DropButton dropButton;
    DTSEditorModuleMgr moduleMgr;
    DTSQuery queryMgr;
    DTSSearchOptions sopts = 
        new DTSSearchOptions(9999, DTSSearchOptions.ALL_NAMESPACES, allCASD);
        
    public TableViewerTablePanel(DTSEditorModuleMgr mgr, DTSModuleConfig cfg) {
        super(new BorderLayout());
        moduleMgr =mgr;
        queryMgr = DTSAppManager.getQuery();
        director = null;								//no active data
        config = new TableViewerConfig(cfg);
        title = new JLabel("");
        JPanel panel = new JPanel(new BorderLayout());
        panel.setBorder(new EmptyBorder(5, 10, 5, 10));
        panel.add(title, "North");
        add(panel, "North");
        viewTable = new TableViewerTable(moduleMgr, this);
        tableModel = (TableViewerSorter)viewTable.getModel();
        viewTable.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
          
        viewTable.getSelectionModel().addListSelectionListener(
             new ListSelectionListener() {
                 //just enable button
                 public void valueChanged(ListSelectionEvent lse) {
                     if ((options&ADD_REMOVE_ROWS)>0) remButton.setEnabled(true);
                     if ((options&NEW_DELETE_CONCEPTS)>0) delButton.setEnabled(true);
                 }
         }
             );
        JScrollPane scroll = new JScrollPane(viewTable);
        add(scroll, "Center");
        JPanel butPanel = new JPanel();
        Insets margin = new Insets(2, 5, 2, 5);		//make smaller buttons
        dropButton = new DropButton("Drop/Add");
        dropButton.setMargin(margin);
        dropButton.setVisible(false);
        dropButton.setEnabled(false);
        dropButton.addActionListener(this);
        remButton = new DropButton("Remove");
        remButton.setMargin(margin);
        remButton.setVisible(false);
        remButton.setEnabled(false);
        remButton.addActionListener(this);
        newButton = new JButton("New Concept");
        newButton.setMargin(margin);
        newButton.setVisible(false);
        newButton.addActionListener(this);
        delButton = new JButton("Delete Concept");
        delButton.setMargin(margin);
        delButton.setVisible(false);
        delButton.setEnabled(false);
        delButton.addActionListener(this);
		findButton = new JButton("Find");
        findButton.setMargin(margin);
		findButton.setVisible(false);
		findButton.setEnabled(false);
		findButton.addActionListener(this);
		clearButton = new JButton("Clear");
        clearButton.setMargin(margin);
		clearButton.setVisible(false);
		clearButton.setEnabled(false);
		clearButton.addActionListener(this);
        exportButton = new JButton("Export");
        exportButton.setMargin(margin);
		exportButton.setVisible(false);
        exportButton.setEnabled(false);
        exportButton.addActionListener(this);
        butPanel.add(dropButton);
        butPanel.add(remButton);
        butPanel.add(newButton);
        butPanel.add(delButton);
        butPanel.add(findButton);
        butPanel.add(clearButton);
        butPanel.add(exportButton);
        add(butPanel, "South");
    }
          
    /** enable/disable the panel */
    public void enableModule(boolean enable) {
        clearTable();
        dropButton.setVisible(false);
        dropButton.setEnabled(false);
        remButton.setVisible(false);
        remButton.setEnabled(false);
        newButton.setVisible(false);
        newButton.setEnabled(false);
        delButton.setVisible(false);
        delButton.setEnabled(false);
		findButton.setVisible(false);
		findButton.setEnabled(false);
		clearButton.setVisible(false);
		clearButton.setEnabled(false);
		exportButton.setVisible(false);
        exportButton.setEnabled(false);
    }
          
    /** get the config object*/
    public TableViewerConfig getConfig() { return config; }
    
    /** return a pointer to the director panel */
    TableViewerTableDirector getDirector() { return director; }
    
    /** return whether table has been loaded */
    boolean isLoaded() { return director!=null; }
          
    /**************************************************/
    
    /** button handler, common routine so we can cancel editing, etc. */
    public void actionPerformed(ActionEvent ae) {
        Object src = ae.getSource();
        //setBusy(true);
        //first cancel any editing
        viewTable.getDefaultEditor(String.class).cancelCellEditing();
        if (src==newButton) {
            clearSelection();
            director.newConcept();
            // updateButtons()handled via TableChangeEvent
        }
        else if (src==delButton) {
            director.deleteConcepts(viewTable.getSelectedRows());
            clearSelection();
            // updateButtons()handled via TableChangeEvent
        }
        else if (src==dropButton) {
            clearSelection();
            director.addRows(dropButton.getDropEvent());
            // updateButtons()handled via TableChangeEvent
        }
        else if (src==remButton) {
            director.removeRows(viewTable.getSelectedRows());
            clearSelection();
            // updateButtons()handled via TableChangeEvent
        }
		else if (src==findButton) {
			edit();
		}
		else if (src==clearButton) {
			director.clear();
            // updateButtons()handled via TableChangeEvent
		}
        else if (src==exportButton) {
            export();
        }
        //setBusy(false);
    }
          
    private void clearSelection() {
        viewTable.clearSelection();
        remButton.setEnabled(false);
        delButton.setEnabled(false);
    }
    
    //update buttons after potential row change
    private void updateButtons() {
    	boolean selection = viewTable.getSelectedRow()>=0;
        if (dropButton.isVisible()) dropButton.setEnabled(true);
        if (remButton.isVisible()) remButton.setEnabled(selection);
        if (newButton.isVisible()) newButton.setEnabled(true);
        if (delButton.isVisible()) delButton.setEnabled(selection);
        boolean rows = viewTable.getRowCount()>0;
		findButton.setEnabled(rows);
        if (clearButton.isVisible()) clearButton.setEnabled(rows);
        exportButton.setEnabled(rows);
    }

    /**************************************************/
    //methods called from directors
    
    /** set the table title */
    void setTitle() {
    	if (isLoaded()) title.setText(director.getDirectorLabel()+": "+director.getActiveName());
    	else title.setText("");
    }
    
    /** clear and inactivate the table */
    void clearTable() {
        activeState = null;
        director = null;				//not loaded
        setTitle();
        clearModels();                  //clear the table
    }
    
    /** clear the current table data - remove rows, reset format */
     void clearTableData() {
     	DTSConcept[] cons = new DTSConcept[0];
     	resetTableState(activeState.key);
     	//now emulate like a "Load"
     	loadTableViewer(director, activeState.key, cons, options);
     }

    /** load data table, called from a TableViewerTableDirector class */
    void loadTableViewer(TableViewerTableDirector dir, String key, 
                         DTSConcept[] cons, int opts) {
        TableColumn column;
        int width;
                
        director = dir;                                 //save our controller
        options = opts;                                 //and the options
        //setTitle();									//title is set separately after success
        //don't listen while clearing/loading   
        DefaultTableColumnModel colModel = 
            (DefaultTableColumnModel)viewTable.getColumnModel();
        int totalWidth = viewTable.getWidth()*5/10;		//leave some room to add columns at end
        //System.out.println("total width="+totalWidth);
        colModel.removeColumnModelListener(this);
        tableModel.removeTableModelListener(this);
        clearModels();      //clear models
        //now set up attribute vector (and get sort state)
        System.out.println("TableViewer.getting state for "+key);
        TableState state = getTableState(key);
        TableViewerAttribute[] attrs = state.attrs;     
        Integer[] widths = state.widths;
        //set up headers
        System.out.println("TableViewer.building column model");
        for (int i=1, n=attrs.length; i<n; i++) {
            //don't load first attribute (Concept)
            column = new TableColumn(i);
            column.setHeaderValue(attrs[i].toString());
            width = widths[i].intValue();
            column.setPreferredWidth((width>0?width:100)*totalWidth/100);
            column.addPropertyChangeListener(this);
            colModel.addColumn(column);
        }
        //set the sort state
        tableModel.cancelSorting();
        tableModel.setColumnIdentifiers(attrs);
        if (state.sortColumn>=0) 
            tableModel.setSortingStatus(state.sortColumn, 
                 state.sortValue.equals(SORT_ASCENDING)?
                     TableViewerSorter.ASCENDING:TableViewerSorter.DESCENDING);
                
        //now load data into model
        System.out.println("TableViewer.getting table data");

        for (int i=0; i<cons.length; i++)
            tableModel.addRow(cons[i]);
                
        System.out.println("TableViewer.completing load");
        colModel.addColumnModelListener(this);
        tableModel.addTableModelListener(this);
                
        if ((options&ADD_REMOVE_ROWS)>0) {
            dropButton.setVisible(true);
            remButton.setVisible(true);
			clearButton.setVisible(true);
       }
        else {
            dropButton.setVisible(false);
            remButton.setVisible(false);            //might be set on row selection
			clearButton.setVisible(false);
        }
        if ((options&NEW_DELETE_CONCEPTS)>0) {
            newButton.setVisible(true);
            delButton.setVisible(true);         //might be set on row selection
        }
        else {
            newButton.setVisible(false);
            delButton.setVisible(false);            //might be set on row selection
        }
        findButton.setVisible(true);
        exportButton.setVisible(true);
        updateButtons();
    }
          
    /** clear internal table models*/
    private void clearModels() {
        System.out.println("TableViewer.clearing table models");
        tableModel.setRowCount(0);                 //clear the table data
        //now build the column model (ignore first data column)
        DefaultTableColumnModel colModel = 
            (DefaultTableColumnModel)viewTable.getColumnModel();
        for (int i=colModel.getColumnCount()-1; i>=0; i--) //clear out old columns
            colModel.removeColumn(colModel.getColumn(i));
    }
          
    //check for existing concept row, called with qualified name
    boolean isRowPresent(String name) {
        return findRowForConcept(name)>=0;
    }
          
    //find a row
    int findRowForConcept(String name) {
        int n = viewTable.getRowCount();
        for (int row=0; row<n; row++)
            if (name.equals(QualifiedObject.getNameForObject(
                                 (DTSConcept)viewTable.getModel()
                                     .getValueAt(row,0)))) return row;
        return -1;
    }
          
    DTSConcept getConceptForRow(int row) {
        return (DTSConcept)viewTable.getModel().getValueAt(row,0);
    }
          
    void addRow(DTSConcept con) {
        ((TableViewerSorter)viewTable.getModel()).addRow(con);
    }
          
    void removeRow(int row) {
        ((DefaultTableModel)viewTable.getModel()).removeRow(row);
    }
          
    /**************************************************/
    /** listen for column width changes and update config file*/
    public void propertyChange(PropertyChangeEvent pce) {
        //System.out.println("pce:"+pce.getPropertyName()+" in "+pce.getSource());
        if (pce.getPropertyName().equals("preferredWidth")) {
        	//don't bog down user interaction
        	SwingUtilities.invokeLater(new Runnable() {
        		public void run() { saveTableState("pc"); };
        	});
        }
    }
          
    /** listen for sorting changes and update config file*/
    public void tableChanged(TableModelEvent tme) {
        //if an update for all columns, check for state change (could be new sort)
        if (tme.getColumn()==TableModelEvent.ALL_COLUMNS) { 
        	if (tme.getLastRow()>((DefaultTableModel)tme.getSource()).getRowCount()) {
        		//add/remove row or sort change
            	//System.out.println("f="+tme.getFirstRow()+",l="+tme.getLastRow()+
                //                  ",c="+tme.getColumn()+",t="+tme.getType());
            	final String mess = "tme "+tme.getLastRow();
            	SwingUtilities.invokeLater(new Runnable() {
            		public void run() { saveTableState(mess); };
            	});
            	//System.out.println("f="+tme.getFirstRow()+",l="+tme.getLastRow()+
                //                 ",c="+tme.getColumn()+",t="+tme.getType());
        	}
        	//update buttons in case of clear/add/rem/del/new
            updateButtons();
        }
    }

    /** table column model listener - to update config file*/
    public void columnAdded(TableColumnModelEvent tcme) {
        //System.out.println("added="+tcme.getFromIndex()+":"+tcme.getToIndex());
    	SwingUtilities.invokeLater(new Runnable() {
    		public void run() { saveTableState("tcme add"); };
    	});
    }
          
    public void columnMoved(TableColumnModelEvent tcme)  {
        if (tcme.getFromIndex()!=tcme.getToIndex()) {
            //System.out.println("moved="+tcme.getFromIndex()+":"+tcme.getToIndex());
        	SwingUtilities.invokeLater(new Runnable() {
        		public void run() { saveTableState("tcme move"); };
        	});
        }
    }
          
    public void columnRemoved(TableColumnModelEvent tcme) {
        //System.out.println("removed="+tcme.getFromIndex()+":"+tcme.getToIndex());
    	SwingUtilities.invokeLater(new Runnable() {
    		public void run() { saveTableState("tcme rem"); };
    	});
    }
          
    public void columnMarginChanged(ChangeEvent ce) { }
          
    public void columnSelectionChanged(ListSelectionEvent lse) { }
          
    /**************************************************/
    //table edit
    
    TableViewerFindDialog findDlg = null;
    
    /** edit the table via a dialog */
    private void edit() {
    	if (findDlg==null) findDlg = new TableViewerFindDialog(this);
    	findDlg.load(activeState.attrs);
    	findDlg.show();
    }

    /**************************************************/
    //export
    JFileChooser chooser = null;
    
    /** export the table to a text file */
    private boolean export() {
        String fext, fname;
                  
        if (chooser == null) {
            chooser = new JFileChooser();
            chooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
            String dir = System.getProperty("user.dir");
            if (dir!=null) chooser.setCurrentDirectory(new File(dir));
            SimpleFileFilter filter =
                new SimpleFileFilter(new String[]{ "txt" }, "Text Files");
            chooser.setFileFilter(filter);
        }
                  
        chooser.setDialogTitle("Export Output File");
        int res = chooser.showSaveDialog(null);
        if (res!=JFileChooser.APPROVE_OPTION) return false; 
        fname = chooser.getSelectedFile().getAbsolutePath();
        fext = getFileExtension(fname);
        //if (!(fext.equals("xml") || fext.equals("txt"))) {
        if (!fext.equals("txt")) {
            moduleMgr.showErrorMessage("Invalid export file extension.");
            return false;
        }
        if (fileExists(fname)) {
            if (JOptionPane.showConfirmDialog(null, 
                                              "Export file exists. Overwrite?", 
                                              "Table Viewer Warning", 
                                              JOptionPane.YES_NO_OPTION)
                !=JOptionPane.YES_OPTION)
                return false;
        }
        //okay to proceed
        System.out.println("TableViewer.exporting ...");
        if (TableViewerExport.export(viewTable, fname)) {
            JOptionPane.showMessageDialog(null, 
                       "Export to '"+fname+"' completed.", 
                       "Table Viewer Message", JOptionPane.INFORMATION_MESSAGE);
            
            return true;          
        }
        moduleMgr.showErrorMessage("Error writing export file.");
        return false;
    }
          
    /**************************************************/
    /** test if a file already exists*/
    private boolean fileExists(String filen) {
        File file = new File(filen);
        return file.exists();
    }
          
    /** get the file extension */
    private String getFileExtension(String filen) {
        int i = filen.lastIndexOf('.');
        if(i>0 && i<filen.length()-1) 
            return filen.substring(i+1).toLowerCase();
        return "";      
    }
        
          
    /**************************************************/
    class TableState {
        String key;
        TableViewerAttribute[] attrs;
        Integer[] widths;
        int sortColumn;
        String sortValue;
                
        TableState(String s) {
            key = s;
            attrs = null;
            widths = null;
            sortColumn = -1;
            sortValue = SORT_NONE;
        }
    }
          
    //attribute methods
          
    private TableState activeState = null;
    private final static String STATE_DELIM = "|";
          
    /** get table state variables for this list key */
    private TableState getTableState(String key) {
        String s, str;
        TableViewerAttribute attr;
        Vector<TableViewerAttribute> attrs = new Vector<TableViewerAttribute>();
        Vector<Integer> widths = new Vector<Integer>();
        TableState state = new TableState(key);
        str = config.getEntry(key);
        if (str==null || str.equals("")) {
            str = getInitialStateString();
            config.addEntry(key, str);
        }
        attrs.add(new TableViewerAttribute("CONCEPT"));     //invisible column
        widths.add(new Integer(0));
        int col = 1;
        StringTokenizer st = new StringTokenizer(str, STATE_DELIM); 
        while (st.hasMoreTokens()) {
            s = st.nextToken();
            attr = new TableViewerAttribute(s.substring(3));
            if (attr.isValid()) {
	            if (!s.substring(0,1).equals(SORT_NONE)) {	//only one sort column possible
	                state.sortColumn = col++;
	                state.sortValue = s.substring(0,1);
	            }
	            widths.add(new Integer(s.substring(1,3)));
	            attrs.add(attr);
            }
            else {
                JOptionPane.showMessageDialog(null, 
                    "Invalid column attribute '"+s.substring(3)+"'.\nColumn ignored.", 
                    "Table Viewer Warning", JOptionPane.WARNING_MESSAGE);
            }
        }
        state.attrs = (TableViewerAttribute[])attrs.toArray
            (new TableViewerAttribute[0]);
        //recompute widths in face of deletion????
        state.widths = (Integer[])widths.toArray(new Integer[0]);
        if (debug) dumpTableState(state);
        activeState = state;
        return state;
    }
    
    /** reset the state of a key */
    private void resetTableState(String key) {
        if (activeState==null) return;
        config.updateEntry(key, getInitialStateString());
    }
    
    /** return the initial value of a table state */
    private String getInitialStateString() {
        return SORT_NONE + "00" + TableViewerAttribute.CONCEPT_NAME;
    }
          
    /**
     * update configuration with new state (column names/%widths) of table
     * also updates activeState (attrs and widths)
     * returns true if config changed
     */
    private boolean saveTableState(String src) {
        TableColumn column;
        String swidth;
        String newval = "";    //new entry value (list of attr names)
                
        if (activeState==null) return false;
        String oldval = config.getEntry(activeState.key);
        TableViewerAttribute[] attrs = new TableViewerAttribute[viewTable.getColumnCount()+1];
        Integer[] widths = new Integer[viewTable.getColumnCount()+1];
        int col = 0;
        int totalWidth = viewTable.getWidth();
        attrs[0] = new TableViewerAttribute("CONCEPT");     //invisible column
        widths[0] = new Integer(0);
        Enumeration<TableColumn> col_enum = viewTable.getColumnModel().getColumns();
        while (col_enum.hasMoreElements()) {
            column = (TableColumn)col_enum.nextElement();
            swidth = (column.getWidth()*100/totalWidth+100) + "";   //width % string
            newval += STATE_DELIM + 
                sortModelToState(tableModel.getSortingStatus(
                   viewTable.convertColumnIndexToModel(col++))) +
                swidth.substring(1,3) + column.getHeaderValue();
            //now update state elements
            attrs[col] = new TableViewerAttribute((String)column.getHeaderValue());
            widths[col] = new Integer(column.getWidth());
        }
        newval = newval.substring(1);
        if (newval.equals(oldval)) return false;
        if (debug) System.out.println("TableViewer.new state='"+newval+"', old state='"+oldval+"'");
        activeState.attrs = attrs;							//update the attributes
        activeState.widths = widths;
        if (debug)dumpTableState(activeState);
        config.updateEntry(activeState.key, newval);        //will save config
        System.out.println("TableViewer.state saved from "+src);
        return true;
    }
          
    private String sortModelToState(int n) 
    {
        if (n==TableViewerSorter.ASCENDING) return SORT_ASCENDING;
        if (n==TableViewerSorter.DESCENDING) return SORT_DESCENDING;
        return SORT_NONE;
    }
          
    private void dumpTableState(TableState state) {
        System.out.println("Table State:");
        System.out.println("  Key = '"+state.key+"'");
        TableViewerAttribute[] attrs = state.attrs;
        for (int i=0; i<attrs.length; i++)
            System.out.println("  "+i+"='"+attrs[i].toString()+
                               (state.sortColumn==i?("' ["+state.sortValue+"]"):"'")+
                               " (."+state.widths[i].intValue()+")");
    }     

    /**************************************************/
    //process a concept event, return number of lines updated
    //only called if 
    int processConceptEvent(ConceptEvent evt) {
    	if (!isLoaded()) return 0;
    	DTSConcept con = evt.getConcept();
        int cnt = viewTable.getRowCount();
        int n=0;
    	if (evt.getEventType()==DataChangeEvent.EVENT_TYPE_NEW) {
    		//must be namespace LOAD in this namespace
    		if ((options&NEW_DELETE_CONCEPTS)>0 &&
    			director.getActiveFunction()==TableViewerDirectorPanel.LOAD &&
    			con.getNamespaceId()==director.getActiveNamespaceId()) {
    				addRow(con);
    				n++;
    		}
    	}
        for (int row=0; row<cnt; row++) {
            if (evt.isConceptAffected((DTSConcept)viewTable.getModel().getValueAt(row,0))) {
            	System.out.println("row "+row+" affected");
            	if (evt.getEventType()==DataChangeEvent.EVENT_TYPE_DELETE) {
            		removeRow(row);
            		return -1;								//can only be one
            	}
            	if (evt.getEventType()==DataChangeEvent.EVENT_TYPE_MODIFY) {
            		//could be association that effects two rows
            		if (con.getId()==((DTSConcept)viewTable.getModel().getValueAt(row,0)).getId())
                		((TableViewerSorter)viewTable.getModel()).updateRow(row, con);
            		else							//other side of association	
            			((TableViewerSorter)viewTable.getModel()).updateRow(row);
            		n++;
            		//keep going as could be more than one row affected
            	}
            }
        }
        return n;
    }
}
